@@ -2,7 +2,7 @@
                           rcontainerobject  -  description
                              -------------------
     begin                : Thu Aug 19 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -20,6 +20,7 @@
 
 #include "../rbackend/rinterface.h"
 #include "robjectlist.h"
+#include "rslotspseudoobject.h"
 #include "rkvariable.h"
 #include "rfunctionobject.h"
 #include "renvironmentobject.h"
@@ -30,7 +31,7 @@
 
 #include "../debug.h"
 
-RContainerObject::RContainerObject (RContainerObject *parent, const QString &name) : RObject (parent, name) {
+RContainerObject::RContainerObject (RObject *parent, const QString &name) : RObject (parent, name) {
 	RK_TRACE (OBJECTS);
 	type = Container;
 	rownames_object = 0;
@@ -60,10 +61,10 @@ RObject *RContainerObject::updateChildStructure (RObject *child, RData *new_data
 			delete child;
 			return 0;
 		} else {
-			int child_index = getIndexOf (child);
+			int child_index = childmap.indexOf (child);
 			RK_ASSERT (child_index >= 0);
 			if (RKGlobals::tracker ()->removeObject (child, 0, true)) {
-				RData *child_name_data = new_data->getStructureVector ()[0];
+				RData *child_name_data = new_data->getStructureVector ()[StoragePositionName];
 				RK_ASSERT (child_name_data->getDataType () == RData::StringVector);
 				RK_ASSERT (child_name_data->getDataLength () >= 1);
 				QString child_name = child_name_data->getStringVector ()[0];
@@ -79,15 +80,15 @@ RObject *RContainerObject::updateChildStructure (RObject *child, RData *new_data
 bool RContainerObject::updateStructure (RData *new_data) {
 	RK_TRACE (OBJECTS);
 	unsigned int data_length = new_data->getDataLength (); 
-	RK_ASSERT (data_length >= 5);
+	RK_ASSERT (data_length >= StorageSizeBasicInfo);
 	RK_ASSERT (new_data->getDataType () == RData::StructureVector);
 
 	if (!RObject::updateStructure (new_data)) return false;
 
-	if (data_length > 5) {
-		RK_ASSERT (data_length == 6);
+	if (data_length > StorageSizeBasicInfo) {
+		RK_ASSERT (data_length == (StorageSizeBasicInfo + 1));
 
-		RData *children_sub = new_data->getStructureVector ()[5];
+		RData *children_sub = new_data->getStructureVector ()[StoragePositionChildren];
 		RK_ASSERT (children_sub->getDataType () == RData::StructureVector);
 		updateChildren (children_sub);
 		updateRowNamesObject ();
@@ -101,9 +102,9 @@ bool RContainerObject::updateStructure (RData *new_data) {
 RObject *RContainerObject::createChildFromStructure (RData *child_data, const QString &child_name, int position) {
 	RK_TRACE (OBJECTS);
 	RK_ASSERT (child_data->getDataType () == RData::StructureVector);
-	RK_ASSERT (child_data->getDataLength () >= 2);		// need to see at least the type at this point
+	RK_ASSERT (child_data->getDataLength () >= (StoragePositionType + 1));		// need to see at least the type at this point
 
-	RData *type_data = child_data->getStructureVector ()[1];
+	RData *type_data = child_data->getStructureVector ()[StoragePositionType];
 	RK_ASSERT (type_data->getDataType () == RData::IntVector);
 	RK_ASSERT (type_data->getDataLength () == 1);
 
@@ -131,12 +132,15 @@ RObject *RContainerObject::createChildFromStructure (RData *child_data, const QS
 		RK_ASSERT (false);
 		return 0;
 	}
-	RKGlobals::tracker ()->addObject (child_object, this, position);
+	RKGlobals::tracker ()->beginAddObject (child_object, this, position);
+	childmap.insert (position, child_object);
+	RKGlobals::tracker ()->endAddObject (child_object, this, position);
 	return child_object;
 }
 
 void RContainerObject::updateChildren (RData *new_children) {
 	RK_TRACE (OBJECTS);
+
 	RK_ASSERT (new_children->getDataType () == RData::StructureVector);
 	unsigned int new_child_count = new_children->getDataLength ();
 
@@ -146,8 +150,8 @@ void RContainerObject::updateChildren (RData *new_children) {
 	for (unsigned int i = 0; i < new_child_count; ++i) {
 		RData *child_data = new_children->getStructureVector ()[i];
 		RK_ASSERT (child_data->getDataType () == RData::StructureVector);
-		RK_ASSERT (child_data->getDataLength () >= 1);
-		RData *child_name_data = child_data->getStructureVector ()[0];
+		RK_ASSERT (child_data->getDataLength () >= (StoragePositionName + 1));
+		RData *child_name_data = child_data->getStructureVector ()[StoragePositionName];
 		RK_ASSERT (child_name_data->getDataType () == RData::StringVector);
 		RK_ASSERT (child_name_data->getDataLength () >= 1);
 		QString child_name = child_name_data->getStringVector ()[0];
@@ -211,24 +215,6 @@ int RContainerObject::numChildren () const {
 	return childmap.size ();
 }
 
-// KDE4: do we need this? Can't we return the RObjectMap?
-RObject **RContainerObject::children () const {
-	RK_TRACE (OBJECTS);
-	RObject **ret = new RObject *[childmap.size ()];
-
-	for (int i = childmap.size () - 1; i >= 0; --i) {
-		ret[i] = childmap[i];
-	}
-	return ret;
-}
-
-void RContainerObject::writeChildMetaData (RCommandChain *chain) {
-	RK_TRACE (OBJECTS);
-	for (int i = childmap.size () - 1; i >= 0; --i) {
-		childmap[i]->writeMetaData (chain);
-	}
-}
-
 RObject *RContainerObject::findChildByName (const QString &name) const {
 	RK_TRACE (OBJECTS);
 
@@ -265,6 +251,7 @@ void RContainerObject::updateRowNamesObject () {
 
 	int childlen = 0;
 	if (!childmap.isEmpty ()) childlen = childmap[0]->getLength ();
+	rownames_object->extendToLength (childlen);	// in case it is being edited
 	rownames_object->dimensions[0] = childlen;
 
 	if (rownames_object->isType (NeedDataUpdate)) {
@@ -272,63 +259,33 @@ void RContainerObject::updateRowNamesObject () {
 	}
 }
 
-int RContainerObject::getIndexOf (RObject *child) const {
+RObject *RContainerObject::findObjects (const QStringList &path, RObjectSearchMap *matches, const QString &op) {
 	RK_TRACE (OBJECTS);
 
-	return childmap.indexOf (child);
-}
+	fetchMoreIfNeeded ();
 
-RObject *RContainerObject::findObject (const QString &name, bool is_canonified) const {
-	RK_TRACE (OBJECTS);
+	if (op != "$") return RObject::findObjects (path, matches, op);
 
-	QString canonified = name;
-	if (!is_canonified) canonified = canonifyName (name);
+	if (path.length () > 1) {
+		RObject* found = findChildByName (path.value (0));
+		if (found) return found->findObjects (path.mid (2), matches, path.value (1));
+	} else {
+		if (!matches) return findChildByName (path.value (0));
 
-	// TODO: there could be objects with "$" in their names!
-	QString current_level = canonified.section (QChar ('$'), 0, 0);
-	QString remainder = canonified.section (QChar ('$'), 1);
-
-	RObject* found = findChildByName (current_level);
-	if (!found) return 0;
-
-	if (remainder.isEmpty ()) return found;
-
-	return (found->findObject (remainder, true));
-}
-
-void RContainerObject::findObjectsMatching (const QString &partial_name, RObjectSearchMap *current_list, bool name_is_canonified) const {
-	RK_TRACE (OBJECTS);
-	RK_ASSERT (current_list);
-
-	QString canonified = partial_name;
-	if (!name_is_canonified) canonified = canonifyName (partial_name);
-
-	// TODO: there could be objects with "$" in their names!
-	QString current_level = canonified.section (QChar ('$'), 0, 0);
-	QString remainder = canonified.section (QChar ('$'), 1);
-
-	if (canonified.endsWith (QChar ('$'))) {
-		RObject* found = findChildByName (current_level);
-		if (found) found->findObjectsMatching (QString (), current_list, true);
-		return;
-	}
-
-	if (remainder.isEmpty ()) {
+		QString partial = path.value (0);
 		for (int i = 0; i < childmap.size (); ++i) {
 			RObject* child = childmap[i];
-			if (child->getShortName ().startsWith (current_level)) {
+			if (partial.isEmpty () || child->getShortName ().startsWith (partial)) {
 				QString base_name = child->getBaseName ();
-				if (current_list->contains (base_name) || irregularShortName (base_name)) {
-					current_list->insert (child->getFullName (), child);
+				if (matches->contains (base_name) || irregularShortName (base_name)) {
+					matches->insert (child->getFullName (), child);
 				} else {
-					current_list->insert (base_name, child);
+					matches->insert (base_name, child);
 				}
 			}
 		}
-	} else {
-		RObject* found = findChildByName (current_level);
-		if (found) found->findObjectsMatching (remainder, current_list, true);
 	}
+	return 0;
 }
 
 RObject *RContainerObject::createPendingChild (const QString &name, int position, bool container, bool data_frame) {
@@ -349,7 +306,9 @@ RObject *RContainerObject::createPendingChild (const QString &name, int position
 
 	if ((position < 0) || (position > childmap.size ())) position = childmap.size ();
 
-	RKGlobals::tracker ()->addObject (ret, this, position);
+	RKGlobals::tracker ()->beginAddObject (ret, this, position);
+	childmap.insert (position, ret);
+	RKGlobals::tracker ()->endAddObject (ret, this, position);
 
 	return ret;
 }
@@ -365,23 +324,18 @@ void RContainerObject::renameChild (RObject *object, const QString &new_name) {
 	object->name = new_name;
 }
 
-void RContainerObject::insertChild (RObject* child, int position) {
-	RK_TRACE (OBJECTS);
-
-	RK_ASSERT (child->getContainer () == this);
-	if ((position < 0) || (position > childmap.size ())) position = childmap.size ();
-	childmap.insert (position, child);
-}
-
 void RContainerObject::removeChildNoDelete (RObject *object) {
 	RK_TRACE (OBJECTS);
 
-	int i = getIndexOf (object);
-	if (i < 0) {
-		RK_ASSERT (false);
-		return;
-	}
-	childmap.removeAt (i);
+	if (!childmap.removeOne (object)) RK_ASSERT (false);
+}
+
+void RContainerObject::insertChild (RObject* child, int position) {
+	RK_TRACE (OBJECTS);
+
+	RK_ASSERT (child->parentObject () == this);
+	if ((position < 0) || (position > childmap.size ())) position = childmap.size ();
+	childmap.insert (position, child);
 }
 
 void RContainerObject::removeChild (RObject *object, bool removed_in_workspace) {
@@ -416,23 +370,6 @@ QString RContainerObject::renameChildCommand (RObject *object, const QString &ne
 	return ("rk.rename.in.container (" + getFullName () + ", \"" + object->getShortName () + "\", \"" + new_name + "\")");
 }
 
-bool RContainerObject::isParentOf (RObject *object, bool recursive) const {
-	RK_TRACE (OBJECTS);
-
-	for (int i = childmap.size () - 1; i >= 0; --i) {
-		RObject *child = childmap[i];
-		if (child == object) {
-			return true;
-		} else if (recursive && child->isContainer ()) {
-			if (static_cast<RContainerObject *>(child)->isParentOf (object, true)) {
-				return true;
-			}
-		}
-	}
-	
-	return false;
-}
-
 QString RContainerObject::validizeName (const QString &child_name, bool unique) const {
 	RK_TRACE (OBJECTS);
 	RK_ASSERT (isType (GlobalEnv) || isInGlobalEnv ());
