@@ -22,13 +22,9 @@
 #include "../rkglobals.h"
 #include "../debug.h"
 
-RFunctionObject::RFunctionObject (RContainerObject *parent, const QString &name) : RObject (parent, name) {
+RFunctionObject::RFunctionObject (RObject *parent, const QString &name) : RObject (parent, name) {
 	RK_TRACE (OBJECTS);
 	type = Function;
-
-	argcount = 0;
-	argnames = 0;
-	argvalues = 0;
 }
 
 RFunctionObject::~RFunctionObject () {
@@ -39,7 +35,7 @@ QString RFunctionObject::printArgs () const {
 	RK_TRACE (OBJECTS);
 
 	QString ret;
-	for (unsigned int i = 0; i < argcount; ++i) {
+	for (int i = 0; i < argnames.size (); ++i) {
 		if (i) ret.append (", ");
 		ret.append (argnames[i]);
 		if (!argvalues[i].isEmpty ()) {
@@ -52,7 +48,7 @@ QString RFunctionObject::printArgs () const {
 
 bool RFunctionObject::updateStructure (RData *new_data) {
 	RK_TRACE (OBJECTS);
-	RK_ASSERT (new_data->getDataLength () >= 5);
+	RK_ASSERT (new_data->getDataLength () >= StorageSizeBasicInfo);
 	RK_ASSERT (new_data->getDataType () == RData::StructureVector);
 
 	if (!RObject::updateStructure (new_data)) return false;
@@ -64,44 +60,23 @@ bool RFunctionObject::updateStructure (RData *new_data) {
 
 bool RFunctionObject::updateArguments (RData *new_data) {
 	RK_TRACE (OBJECTS);
-	RK_ASSERT (new_data->getDataLength () == 7);
+	RK_ASSERT (new_data->getDataLength () == (StoragePositionFunValues + 1));
 	RK_ASSERT (new_data->getDataType () == RData::StructureVector);
 
-	RData *argnames_data = new_data->getStructureVector ()[5];
-	RData *argvalues_data = new_data->getStructureVector ()[6];
+	RData *argnames_data = new_data->getStructureVector ()[StoragePositionFunArgs];
+	RData *argvalues_data = new_data->getStructureVector ()[StoragePositionFunValues];
 
-	unsigned int new_arglen = argnames_data->getDataLength (); 
 	RK_ASSERT (argnames_data->getDataType () == RData::StringVector);
 	RK_ASSERT (argvalues_data->getDataType () == RData::StringVector);
 
-	RK_ASSERT (new_arglen == argvalues_data->getDataLength ());
-	QString *new_argnames = argnames_data->getStringVector ();
-	QString *new_argvalues = argvalues_data->getStringVector ();
-	argnames_data->detachData ();
-	argvalues_data->detachData ();
+	QStringList new_argnames = argnames_data->getStringVector ();
+	QStringList new_argvalues = argvalues_data->getStringVector ();
+	RK_ASSERT (new_argnames.size () == new_argvalues.size ());
 
-	bool changed = false;
-	if (new_arglen != argcount) {
-		changed = true;
-	} else {
-		for (unsigned int i = 0; i < new_arglen; ++i) {
-			if (argnames[i] != new_argnames[i]) {
-				changed = true;
-				break;
-			}
-
-			if (argvalues[i] != new_argvalues[i]) {
-				changed = true;
-				break;
-			}
-		}
+	if ((new_argnames != argnames) || (new_argvalues != argvalues)) {
+		argnames = new_argnames;
+		argvalues = new_argvalues;
+		return true;
 	}
-
-	argcount = new_arglen;
-	delete [] argnames;
-	delete [] argvalues;
-	argnames = new_argnames;
-	argvalues = new_argvalues;
-
-	return changed;
+	return false;
 }
