@@ -2,7 +2,7 @@
                           rkmodificationtracker  -  description
                              -------------------
     begin                : Tue Aug 31 2004
-    copyright            : (C) 2004, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2007, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -98,6 +98,13 @@ public:
 	/** reimplemented from  QAbstractItemModel::headerData() to provide column names */
 	QVariant headerData (int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
 
+	/** reimplemented from  QAbstractItemModel::canFetchMore() */
+	bool canFetchMore (const QModelIndex &parent) const;
+	/** reimplemented from  QAbstractItemModel::fetchMore() */
+	void fetchMore (const QModelIndex &parent);
+	/** reimplemented from  QAbstractItemModel::hasChildren() */
+	bool hasChildren (const QModelIndex &parent = QModelIndex ()) const; 
+
 	/** convenience function to create an index for a particular object */
 	QModelIndex indexFor (RObject *object) const;
 };
@@ -125,7 +132,7 @@ public:
 /** recursive! */
 	void lockUpdates (bool lock);
 /** returns (the first) editor that is currently active for this object, or 0, if there is no editor */
-	RKEditor* objectEditor (RObject* object);
+	RKEditor* objectEditor (const RObject* object);
 private:
 	int updates_locked;
 /** relay change notifications to connected listeners. This is not pretty, since the arguments change their meanings depending on the type of notification, but for now this is ok */
@@ -137,9 +144,12 @@ friend class RObjectListener;
 	QMultiHash<RObject*, RObjectListener*> listeners;
 
 friend class RContainerObject;
+friend class REnvironmentObject;
+friend class RObject;
 friend class RObjectList;
 /** essentially like the above function(s). All objects listening for child additions on the parent will be notified */
-	void addObject (RObject *object, RContainerObject* parent, int position);
+	void beginAddObject (RObject *object, RObject* parent, int position);
+	void endAddObject (RObject *object, RObject* parent, int position);
 /** essentially like the above function(s). All objects listening for child position changed on the parent will be notified */
 	void moveObject (RContainerObject *parent, RObject* child, int old_index, int new_index);
 };
