@@ -2,7 +2,7 @@
                           rkvariable  -  description
                              -------------------
     begin                : Thu Aug 12 2004
-    copyright            : (C) 2004, 2007, 2008, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2007, 2008, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -19,7 +19,6 @@
 #include <qstringlist.h>
 #include "float.h"
 #include "math.h"
-#include "limits.h"
 
 #include "rcontainerobject.h"
 #include "robjectlist.h"
@@ -34,9 +33,6 @@
 
 #include "../debug.h"
 
-QString *RKVariable::na_char = new QString ("");
-QString *RKVariable::unknown_char = new QString ("?");
-
 RKVariable::RKVariable (RContainerObject *parent, const QString &name) : RObject (parent, name) {
 	RK_TRACE (OBJECTS);
 	type = Variable;
@@ -62,9 +58,11 @@ void RKVariable::setVarType (RObject::RDataType new_type, bool sync) {
 	if (data) {
 		// quick and dirty approach! TODO: make more efficient
 		QStringList list;
-		for (int i=0; i < getLength (); ++i) {
-			list.append (getText (i));
-		}
+#if QT_VERSION >= 0x040700
+		list.reserve (getLength ());
+#endif
+		bool labelled = (new_type == DataCharacter);
+		for (int i=0; i < getLength (); ++i) list.append (getText (i, labelled));
 
 		// all pending changes are moot
 		discardUnsyncedChanges ();
@@ -88,11 +86,8 @@ void RKVariable::setVarType (RObject::RDataType new_type, bool sync) {
 
 		// re-set all data
 		lockSyncing (true);
-		int i = 0;
-		for (QStringList::const_iterator it = list.constBegin (); it != list.constEnd (); ++it) {
-			setText (i, *it);
-			i++;
-		}
+		for (int i = list.size () - 1; i >= 0; --i) setText (i, list[i]);
+
 		if (sync) {
 			QString command = ".rk.set.vector.mode(" + getFullName () + ", ";
 			if (new_type == RObject::DataCharacter) command += "as.character";
@@ -101,9 +96,10 @@ void RKVariable::setVarType (RObject::RDataType new_type, bool sync) {
 			else if (new_type == RObject::DataFactor) command += "as.factor";
 			command += ")";
 			RKGlobals::rInterface ()->issueCommand (command, RCommand::App | RCommand::Sync, QString::null);
+			if (new_type == RObject::DataFactor) writeValueLabels (0);		// as.factor resets the "levels"-attribute!
 
 			syncDataToR ();
-		}
+		} else discardUnsyncedChanges ();
 		lockSyncing (false);
 	} else {
 		setDataType (new_type);
@@ -158,13 +154,13 @@ void RKVariable::rCommandDone (RCommand *command) {
 			setNumericFromR (0, getLength () - 1, cdata->getRealVector ());
 		} else if (cdata->getDataType () == RData::IntVector) {
 			unsigned int len = getLength ();
-			double *dd = new double[len];
+			QVector<double> dd;
+			dd.reserve (len);
 			for (unsigned int i = 0; i < len; ++i) {
-				if (cdata->getIntVector ()[i] == INT_MIN) dd[i] = NAN;
-				else dd[i] = (double) cdata->getIntVector ()[i];
+				if (RInterface::isNaInt (cdata->getIntVector ()[i])) dd.append (NAN);
+				else dd.append ((double) cdata->getIntVector ()[i]);
 			}
 			setNumericFromR (0, getLength () - 1, dd);
-			delete [] dd;
 		}
 
 		// now set the invalid fields (only if they are still NAs in the R data)
@@ -206,7 +202,7 @@ void RKVariable::rCommandDone (RCommand *command) {
 void RKVariable::setLength (int len) {
 	RK_TRACE (OBJECTS);
 	RK_ASSERT (!getLength ());	// should only be called once
-	RK_ASSERT (dimensions);
+	RK_ASSERT (!dimensions.isEmpty ());
 
 	dimensions[0] = len;
 }
@@ -218,6 +214,10 @@ bool RKVariable::updateType (RData *new_data) {
 		int old_type = type;
 		bool ret = RObject::updateType (new_data);
 		int new_type = type;
+
+		// Convert old values to the new data type.
+		// TODO: This is quite inefficient, as we will update the data from R in a second, anyway.
+		// Still it is a quick, dirty, and safe way to keep the data representation in a suitable format
 		type = old_type;		// needed to read out the old data
 		setVarType (typeToDataType (new_type), false);
 		type = new_type;
@@ -384,9 +384,10 @@ void RKVariable::cellsChanged (int from_row, int to_row) {
 }
 
 void RKVariable::extendToLength (int length) {
+	if (!data) return;
 	RK_TRACE (OBJECTS);
 
-	if (length <= 0) length = 1;
+	if (length <= 0) length = 0;
 	int old_length = getLength ();
 	if (length <= old_length) return;
 
@@ -403,7 +404,7 @@ void RKVariable::extendToLength (int length) {
 QString RKVariable::getText (int row, bool pretty) const {
 	if (row >= getLength ()) {
 		RK_ASSERT (false);
-		return (*unknown_char);
+		return (QString ());
 	}
 
 	if (data->cell_states[row] & RKVarEditData::Invalid) {
@@ -411,16 +412,9 @@ QString RKVariable::getText (int row, bool pretty) const {
 		return (data->invalid_fields.value (row));
 	}
 
-	if (data->cell_states[row] & RKVarEditData::NA) {
-		return (*na_char);
-	}
+	if (data->cell_states[row] & RKVarEditData::NA) return (QString ());
 
-	if (pretty && (data->value_labels)) {
-		QString otext = getText (row);
-		if (data->value_labels->contains (otext)) {
-			return (*(data->value_labels))[otext];
-		}
-	}
+	if (pretty) return (getLabeled (row));
 
 	if (getDataType () == DataCharacter) {
 		RK_ASSERT (!data->cell_strings.isEmpty ());
@@ -445,7 +439,7 @@ QString RKVariable::getRText (int row) const {
 	if ((cell_state == ValueUnused) || (cell_state == ValueInvalid)) {
 		return ("NA");
 	} else if (getDataType () == DataFactor) {
-		return (rQuote (getLabeled (row)));
+		return (rQuote (getText (row, true)));
 	} else if (getDataType () == DataCharacter) {
 		return (rQuote (getText (row)));
 	} else if (getDataType () == DataLogical) {
@@ -470,53 +464,59 @@ void RKVariable::setText (int row, const QString &text) {
 		data->cell_states[row] = 0;
 	}
 
+	bool valid = true;
 	if (text.isNull ()) {
 		data->cell_states[row] |= RKVarEditData::NA;
+	} else if (text.isEmpty () && getDataType () != DataCharacter) {
+		data->cell_states[row] |= RKVarEditData::NA;
 	} else {
 		if (getDataType () == DataCharacter) {
 			data->cell_strings[row] = text;
-			data->cell_states[row] |= RKVarEditData::Valid;
 		} else if (getDataType () == DataFactor) {
-			if (text.isEmpty ()) {
-				data->cell_states[row] |= RKVarEditData::NA;
-			} else if (data->value_labels && data->value_labels->contains (text)) {
-				data->cell_doubles[row] = text.toInt ();
-				data->cell_states[row] |= RKVarEditData::Valid;
-			} else {
-				data->invalid_fields.insert (row, text);
-				data->cell_states[row] |= RKVarEditData::Invalid | RKVarEditData::UnsyncedInvalidState;
+			if (data->value_labels) {
+				QString realtext = data->value_labels->key (text);
+				if (realtext.isEmpty ()) valid = false;
+				else data->cell_doubles[row] = realtext.toInt ();
+			} else valid = false;
+			if (!valid) {	// setting by lavel failed. Try to set a numeric value, instead
+				data->cell_doubles[row] = text.toDouble (&valid);
 			}
+		} else if (getDataType () == DataLogical) {
+			if (text == "0" || text == "F" || text == "FALSE") data->cell_doubles[row] = 0;
+			else if (text == "1" || text == "T" || text == "TRUE") data->cell_doubles[row] = 1;
+			else valid = false;
 		} else {
-			bool ok;
-			if (text.isEmpty ()) {
-				data->cell_states[row] |= RKVarEditData::NA;
-			} else {
-				data->cell_doubles[row] = text.toDouble (&ok);
-				if (ok) {
-					data->cell_states[row] |= RKVarEditData::Valid;
-				} else {
-					data->invalid_fields.insert (row, text);
-					data->cell_states[row] |= RKVarEditData::Invalid | RKVarEditData::UnsyncedInvalidState;
-				}
-			}
+			data->cell_doubles[row] = text.toDouble (&valid);
 		}
 	}
+
+	if (valid) {
+		if (!(data->cell_states[row] & RKVarEditData::NA)) data->cell_states[row] |= RKVarEditData::Valid;
+	} else {
+		data->invalid_fields.insert (row, text);
+		data->cell_states[row] |= RKVarEditData::Invalid | RKVarEditData::UnsyncedInvalidState;
+	}
+
 	cellsChanged (row, row);
 }
 
 QString RKVariable::getLabeled (int row) const {
-	if (data->value_labels) {
-		QString otext = getText (row);
+	QString otext = getText (row);
+	if (getDataType () == DataLogical) {
+		if (otext == "0") return "FALSE";
+		else if (otext == "1") return "TRUE";
+	} else if (data->value_labels) {
 		if (data->value_labels->contains (otext)) {
 			return (*(data->value_labels))[otext];
 		}
 	}
-	return getText (row);
+	return otext;
 }
 
-void RKVariable::setNumericFromR (int from_row, int to_row, double *numdata) {
+void RKVariable::setNumericFromR (int from_row, int to_row, const QVector<double> &numdata) {
 	RK_TRACE (OBJECTS);
 	RK_ASSERT (to_row < getLength ());
+	RK_ASSERT ((to_row - from_row) < numdata.size ());
 
 	if (getDataType () == DataCharacter) {
 		RK_ASSERT (false);		// asserting false to catch cases of this use for now. it's not really a problem, though
@@ -575,9 +575,10 @@ QString *RKVariable::getCharacter (int from_row, int to_row) const {
 	return ret;
 }
 
-void RKVariable::setCharacterFromR (int from_row, int to_row, QString *txtdata) {
+void RKVariable::setCharacterFromR (int from_row, int to_row, const QStringList &txtdata) {
 	RK_TRACE (OBJECTS);
 	RK_ASSERT (to_row < getLength ());
+	RK_ASSERT ((to_row - from_row) < txtdata.size ());
 
 	lockSyncing (true);
 	int i=0;
