@@ -2,7 +2,7 @@
                           robject  -  description
                              -------------------
     begin                : Thu Aug 19 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,9 +22,11 @@
 #include <klocale.h>
 
 #include "../rbackend/rinterface.h"
+#include "../rbackend/rkrbackendprotocol_shared.h"
 #include "../rkglobals.h"
 #include "robjectlist.h"
 #include "rcontainerobject.h"
+#include "rslotspseudoobject.h"
 #include "rkvariable.h"
 #include "renvironmentobject.h"
 #include "rfunctionobject.h"
@@ -33,26 +35,26 @@
 
 #include "../debug.h"
 
-RObject::RObject (RContainerObject *parent, const QString &name) {
+namespace RObjectPrivate {
+	QVector<qint32> dim_null (1, 0);
+}
+
+RObject::RObject (RObject *parent, const QString &name) {
 	RK_TRACE (OBJECTS);
-	
+
 	RObject::parent = parent;
 	RObject::name = name;
 	type = 0;
 	meta_map = 0;
-	classnames = 0;
-	num_classes = 0;
-	dimensions = new int[1];	// safe initialization
-	dimensions[0] = 0;
-	num_dimensions = 0;
+	slots_pseudo_object = 0;
+	dimensions = RObjectPrivate::dim_null;	// safe initialization
 }
 
 RObject::~RObject () {
 	RK_TRACE (OBJECTS);
 
-	delete [] dimensions;
-	delete [] classnames;
 	cancelOutstandingCommands ();
+	delete slots_pseudo_object;
 }
 
 bool RObject::irregularShortName (const QString &name) {
@@ -76,34 +78,26 @@ QString RObject::getLabel () const {
 	return getMetaProperty ("label");
 }
 
-RObject *RObject::findObject (const QString &, bool) const {
+RObject* RObject::findObjects (const QStringList &path, RObjectSearchMap *matches, const QString &op) {
 	RK_TRACE (OBJECTS);
+	// not a container
+	if (op == "@") {
+		if (slots_pseudo_object) return (slots_pseudo_object->findObjects (path, matches, "$"));
+	}
 	return 0;
 }
 
-void RObject::findObjectsMatching (const QString &, RObjectSearchMap *, bool) const {
-	RK_TRACE (OBJECTS);
-	return;
-}
-
 QString RObject::getMetaProperty (const QString &id) const {
 	RK_TRACE (OBJECTS);
-	if (meta_map) {
-		RObject::MetaMap::iterator it;
-		if ((it = meta_map->find (id)) != meta_map->end ()) {
-			return (it.value ());
-		}
-	}
+	if (meta_map) return (meta_map->value (id));
 	return QString ();
 }
 
 QString RObject::getDescription () const {
 	RK_TRACE (OBJECTS);
 	if (meta_map) {
-		RObject::MetaMap::iterator it;
-		if ((it = meta_map->find ("label")) != meta_map->end ()) {
-			return (getShortName () + " (" + it.value () + ')');
-		}
+		QString label = meta_map->value ("label");
+		if (!label.isEmpty ()) return (getShortName () + " (" + label + ')');
 	}
 	return getShortName ();;
 }
@@ -139,11 +133,11 @@ QString RObject::getObjectDescription () const {
 	}
 
 	if (isType (Container | Variable)) {
-		if (num_dimensions == 1) {
-			ret.append ("<br><b>" + i18n ("Length: ") + QString::number (dimensions[0]));
-		} else if (num_dimensions > 1) {
-			ret.append ("<br><b>" + i18n ("Dimensions: "));
-			for (unsigned int i=0; i < num_dimensions; ++i) {
+		if (dimensions.size () == 1) {
+			ret.append ("<br><b>" + i18n ("Length: ") + " </b>"  + QString::number (dimensions[0]));
+		} else if (dimensions.size () > 1) {
+			ret.append ("<br><b>" + i18n ("Dimensions: ") + " </b>");
+			for (int i=0; i < dimensions.size (); ++i) {
 				if (i) ret.append (", ");
 				ret.append (QString::number (dimensions[i]));
 			}
@@ -163,53 +157,28 @@ void RObject::setLabel (const QString &value, bool sync) {
 void RObject::setMetaProperty (const QString &id, const QString &value, bool sync) {
 	RK_TRACE (OBJECTS);
 	if (value.isEmpty ()) {
-		if (meta_map) {
-			meta_map->remove (id);
-			if (!meta_map->size ()) {
-				delete meta_map;
-				meta_map = 0;
-				type -= (type & HasMetaObject);
-			}
-		}
-		RKGlobals::tracker ()->objectMetaChanged (this);
-		return;
-	}
-	
-	if (meta_map) {
-		RObject::MetaMap::iterator it;
-		if ((it = meta_map->find (id)) != meta_map->end ()) {
-			if (it.value () == value) return;
-		}
+		if (meta_map && meta_map->contains (id)) meta_map->remove (id);
+		else return;
 	} else {
-		meta_map = new MetaMap;
+		if (!meta_map) meta_map = new MetaMap;
+		else if (meta_map->value (id) == value) return;
+
+		meta_map->insert (id, value);
 	}
 
-	meta_map->insert (id, value);
 	if (sync) writeMetaData (0);
 	RKGlobals::tracker ()->objectMetaChanged (this);
 }
 
 QString RObject::makeClassString (const QString &sep) const {
 	RK_TRACE (OBJECTS);
-	QString ret;
-	bool first = true;
-	for (unsigned int i=0; i < numClasses (); ++i) {
-		if (first) first = false;
-		else ret.append (sep);
-		ret.append (getClassName (i));
-	}
-	return ret;
+	return (classnames.join (sep));
 }
 
 bool RObject::inherits (const QString &class_name) const {
 	RK_TRACE (OBJECTS);
 
-	for (unsigned int i=0; i < numClasses (); ++i) {
-		if (getClassName (i) == class_name) {
-			return true;
-		}
-	}
-	return false;
+	return (classnames.contains (class_name));
 }
 
 QString RObject::makeChildName (const QString &short_child_name, bool) const {
@@ -224,36 +193,32 @@ QString RObject::makeChildBaseName (const QString &short_child_name) const {
 
 void RObject::writeMetaData (RCommandChain *chain) {
 	RK_TRACE (OBJECTS);
-	
-	if (!meta_map) {
-		if (hasMetaObject ()) {
-			RCommand *command = new RCommand ("attr (" + getFullName () + ", \".rk.meta\") <- NULL", RCommand::App | RCommand::Sync);
-			RKGlobals::rInterface ()->issueCommand (command, chain);
+
+	if (!meta_map) return;
+
+	QString map_string;
+	if (meta_map->isEmpty ()) {
+		map_string.append ("NULL");
+
+		delete meta_map;	// now that is is synced, delete it
+		meta_map = 0;
+	} else {
+		for (MetaMap::const_iterator it = meta_map->constBegin (); it != meta_map->constEnd (); ++it) {
+			if (!map_string.isEmpty ()) map_string.append (", ");
+			map_string.append (rQuote (it.key ()) + '=' + rQuote (it.value ()));
 		}
-		type -= (type & HasMetaObject);
-		return;
+		map_string = "c (" + map_string + ')';
 	}
-	
-	QString command_string = ".rk.set.meta (" + getFullName () + ", c (";
-	for (MetaMap::const_iterator it = meta_map->constBegin (); it != meta_map->constEnd (); ++it) {
-		if (it != meta_map->constBegin ()) {
-			command_string.append (", ");
-		}
-		command_string.append (rQuote (it.key ()) + '=' + rQuote (it.value ()));
-	}
-	command_string.append ("))");
-	
-	RCommand *command = new RCommand (command_string, RCommand::App | RCommand::Sync);
+
+	RCommand *command = new RCommand (".rk.set.meta (" + getFullName () + ", " + map_string + ')', RCommand::App | RCommand::Sync);
 	RKGlobals::rInterface ()->issueCommand (command, chain);
-	
-	type |= HasMetaObject;
 }
 
 void RObject::updateFromR (RCommandChain *chain) {
 	RK_TRACE (OBJECTS);
 
 	RCommand *command;
-	if (getContainer () == RObjectList::getGlobalEnv ()) {
+	if (parentObject () == RObjectList::getGlobalEnv ()) {
 #warning TODO: find a generic solution
 // We handle objects directly in .GlobalEnv differently. That's to avoid forcing promises, when addressing the object directly. In the long run, .rk.get.structure should be reworked to simply not need the value-argument in any case.
 		 command = new RCommand (".rk.get.structure.global (" + rQuote (getShortName ()) + ')', RCommand::App | RCommand::Sync | RCommand::GetStructuredData, QString::null, this, ROBJECT_UDPATE_STRUCTURE_COMMAND);
@@ -263,6 +228,26 @@ void RObject::updateFromR (RCommandChain *chain) {
 		command = new RCommand (".rk.get.structure (" + getFullName () + ", " + rQuote (getShortName ()) + ')', RCommand::App | RCommand::Sync | RCommand::GetStructuredData, QString::null, this, ROBJECT_UDPATE_STRUCTURE_COMMAND);
 	}
 	RKGlobals::rInterface ()->issueCommand (command, chain);
+
+	type |= Updating;	// will be cleared, implicitly, when the new structure gets set
+}
+
+void RObject::fetchMoreIfNeeded (int levels) {
+	RK_TRACE (OBJECTS);
+
+	if (isType (Updating)) return;
+	if (isType (Incomplete)) {
+		updateFromR (0);
+		return;
+	}
+	if (slots_pseudo_object) slots_pseudo_object->fetchMoreIfNeeded (levels);
+	// Note: We do NOT do the same for any namespaceEnvironment, deliberately
+	if (levels <= 0) return;
+	if (!isContainer ()) return;
+	const RObjectMap children = static_cast<RContainerObject*> (this)->childmap;
+	foreach (RObject* child, children) {
+		child->fetchMoreIfNeeded (levels - 1);
+	}
 }
 
 void RObject::rCommandDone (RCommand *command) {
@@ -275,8 +260,8 @@ void RObject::rCommandDone (RCommand *command) {
 			RKGlobals::tracker ()->removeObject (this, 0, true);
 			return;
 		}
-		if (parent) parent->updateChildStructure (this, command);		// this may result in a delete, so nothing after this!
-		else updateStructure (command);		// if we have no parent, likely we're the RObjectList 
+		if (parent && parent->isContainer ()) static_cast<RContainerObject*> (parent)->updateChildStructure (this, command);		// this may result in a delete, so nothing after this!
+		else updateStructure (command);		// no (container) parent can happen for RObjectList and pseudo objects
 		return;
 	} else {
 		RK_ASSERT (false);
@@ -289,23 +274,31 @@ bool RObject::updateStructure (RData *new_data) {
 		return false;
 	}
 
-	RK_ASSERT (new_data->getDataLength () >= 5);
+	RK_ASSERT (new_data->getDataLength () >= StorageSizeBasicInfo);
 	RK_ASSERT (new_data->getDataType () == RData::StructureVector);
 
 	if (!canAccommodateStructure (new_data)) return false;
 
 	bool properties_change = false;
 
-	properties_change = updateName (new_data->getStructureVector ()[0]);
-	properties_change = updateType (new_data->getStructureVector ()[1]);
-	properties_change = updateClasses (new_data->getStructureVector ()[2]);
-	properties_change = updateMeta (new_data->getStructureVector ()[3]);
-	properties_change = updateDimensions (new_data->getStructureVector ()[4]);
+	properties_change = updateName (new_data->getStructureVector ()[StoragePositionName]);
+	properties_change = updateType (new_data->getStructureVector ()[StoragePositionType]);
+	properties_change = updateClasses (new_data->getStructureVector ()[StoragePositionClass]);
+	properties_change = updateMeta (new_data->getStructureVector ()[StoragePositionMeta]);
+	properties_change = updateDimensions (new_data->getStructureVector ()[StoragePositionDims]);
+	properties_change = updateSlots (new_data->getStructureVector ()[StoragePositionSlots]);
 
 	if (properties_change) RKGlobals::tracker ()->objectMetaChanged (this);
 	if (type & NeedDataUpdate) updateDataFromR (0);
 	if (isPending ()) type -= Pending;
 
+	if (type & Incomplete) {
+		// If the (new!) type is "Incomplete", it means, the structure getter simply stopped at this point.
+		// In case we already have child info, we should update it (TODO: perhaps only, if anything is listening for child objects?)
+		if (numChildrenForObjectModel () && (!isType (Updating))) updateFromR (0);
+		return true;
+	}
+
 	return true;
 }
 
@@ -332,11 +325,11 @@ void RObject::markDataDirty () {
 
 bool RObject::canAccommodateStructure (RData *new_data) {
 	RK_TRACE (OBJECTS);
-	RK_ASSERT (new_data->getDataLength () >= 5);
+	RK_ASSERT (new_data->getDataLength () >= StorageSizeBasicInfo);
 	RK_ASSERT (new_data->getDataType () == RData::StructureVector);
 
-	if (!isValidName (new_data->getStructureVector ()[0])) return false;
-	if (!isValidType (new_data->getStructureVector ()[1])) return false;
+	if (!isValidName (new_data->getStructureVector ()[StoragePositionName])) return false;
+	if (!isValidType (new_data->getStructureVector ()[StoragePositionType])) return false;
 	return true;
 }
 
@@ -385,6 +378,7 @@ bool RObject::updateType (RData *new_data) {
 
 	bool changed = false;
 	int new_type = new_data->getIntVector ()[0];
+	if (type & PseudoObject) new_type |= PseudoObject;
 	if (type & Misplaced) new_type |= Misplaced;
 	if (type & Pending) new_type |= Pending;	// NOTE: why don't we just clear the pending flag, here? Well, we don't want to generate a change notification for this. TODO: rethink the logic, and maybe use an appropriate mask
 	if (type & NeedDataUpdate) new_type |= NeedDataUpdate;
@@ -402,25 +396,12 @@ bool RObject::updateClasses (RData *new_data) {
 
 	bool change = false;
 
-	unsigned int new_len = new_data->getDataLength ();
-	QString *new_classes = new_data->getStringVector ();
-	new_data->detachData ();
-
-	if (numClasses () != new_len) {
+	QStringList new_classes = new_data->getStringVector ();
+	if (new_classes != classnames) {
 		change = true;
-	} else {
-		for (unsigned int cn=0; cn < numClasses (); ++cn) {
-			if (classnames[cn] != new_classes[cn]) {
-				change = true;
-				break;
-			}
-		}
+		classnames = new_classes;
 	}
 
-	num_classes = new_len;
-	delete [] classnames;
-	classnames = new_classes;
-
 	return change;
 }
 
@@ -430,11 +411,10 @@ bool RObject::updateMeta (RData *new_data) {
 	RK_ASSERT (new_data->getDataType () == RData::StringVector);
 
 	unsigned int len = new_data->getDataLength ();
-	if (len == 1) len = 0;		// if it's a single element, it's just a dummy
 	bool change = false;
 	if (len) {
 		if (!meta_map) meta_map = new MetaMap;
-		meta_map->clear ();
+		else meta_map->clear ();
 
 		RK_ASSERT (!(len % 2));
 		unsigned int cut = len/2;
@@ -442,7 +422,6 @@ bool RObject::updateMeta (RData *new_data) {
 			meta_map->insert (new_data->getStringVector ()[i], new_data->getStringVector ()[i+cut]);
 		}
 
-		type |= HasMetaObject;
 		// TODO: only signal change, if there really was a change!
 		change = true;
 	} else {		// no meta data received
@@ -451,8 +430,6 @@ bool RObject::updateMeta (RData *new_data) {
 			meta_map = 0;
 			change = true;
 		}
-
-		type -= (type & HasMetaObject);
 	}
 	return change;
 }
@@ -462,37 +439,122 @@ bool RObject::updateDimensions (RData *new_data) {
 	RK_ASSERT (new_data->getDataLength () >= 1);
 	RK_ASSERT (new_data->getDataType () == RData::IntVector);
 
-	bool changed = false;
-
-	unsigned int new_len = new_data->getDataLength ();
-	int *new_dimensions = new_data->getIntVector ();
-	new_data->detachData ();
-
-	if (num_dimensions != new_len) {
-		changed = true;
-	} else {
-		for (unsigned int d=0; d < num_dimensions; ++d) {
-			if (dimensions[d] != new_dimensions[d]) {
-				changed = true;
-				break;
+	QVector<qint32> new_dimensions = new_data->getIntVector ();
+	if (new_dimensions != dimensions) {
+		if (new_dimensions.isEmpty ()) {
+			if (dimensions != RObjectPrivate::dim_null) {
+				dimensions = RObjectPrivate::dim_null;
+				return (true);
 			}
+		} else {
+#warning TODO: ugly hack. Should be moved to RKVariable, somehow.
+			if (type & Variable) static_cast<RKVariable*> (this)->extendToLength (new_dimensions[0]);
+
+			dimensions = new_dimensions;
+			return (true);
 		}
 	}
-	delete [] dimensions;
-	num_dimensions = new_len;
-	dimensions = new_dimensions;
+	return (false);
+}
 
-	return true;
+bool RObject::updateSlots (RData *new_data) {
+	RK_TRACE (OBJECTS);
+
+	if (new_data->getDataLength ()) {
+		RK_ASSERT (new_data->getDataType () == RData::StructureVector);
+		bool added = false;
+		if (!slots_pseudo_object) {
+			slots_pseudo_object = new RSlotsPseudoObject (this, QString ());
+			added = true;
+			RKGlobals::tracker ()->lockUpdates (true);
+		}
+		bool ret = slots_pseudo_object->updateStructure (new_data->getStructureVector ()[0]);
+		if (added) {
+			RKGlobals::tracker ()->lockUpdates (false);
+
+			int index = getObjectModelIndexOf (slots_pseudo_object);
+			RSlotsPseudoObject *spo = slots_pseudo_object;
+			slots_pseudo_object = 0;	// HACK: Must not be included in the count during the call to beginAddObject
+			RKGlobals::tracker ()->beginAddObject (spo, this, index);
+			slots_pseudo_object = spo;
+			RKGlobals::tracker ()->endAddObject (spo, this, index);
+		}
+		return ret;
+	} else if (slots_pseudo_object) {
+		RKGlobals::tracker ()->removeObject (slots_pseudo_object, 0, true);
+	}
+	return false;
+}
+
+int RObject::getObjectModelIndexOf (RObject *child) const {
+	RK_TRACE (OBJECTS);
+
+	int offset = 0;
+	if (isContainer ()) {
+		int pos = static_cast<const RContainerObject*> (this)->childmap.indexOf (child);
+		if (pos >= 0) return pos + offset;
+		offset += static_cast<const RContainerObject*> (this)->childmap.size ();
+	}
+	if (slots_pseudo_object) {
+		if (child == slots_pseudo_object) return offset;
+		offset += 1;
+	}
+	if (isType (Environment) && static_cast<const REnvironmentObject*> (this)->namespaceEnvironment ()) {
+		if (child == static_cast<const REnvironmentObject*> (this)->namespaceEnvironment ()) return offset;
+		offset += 1;
+	}
+	return -1;
+}
+
+int RObject::numChildrenForObjectModel () const {
+	RK_TRACE (OBJECTS);
+
+	int ret = isContainer () ? static_cast<const RContainerObject*>(this)->numChildren () : 0;
+	if (slots_pseudo_object) ret += 1;
+	if (isType (PackageEnv) && static_cast<const REnvironmentObject*>(this)->namespaceEnvironment ()) ret += 1;
+	return ret;
+}
+
+RObject *RObject::findChildByObjectModelIndex (int index) const {
+	int offset = index;
+	if (isContainer ()) {
+		const RContainerObject *container = static_cast<const RContainerObject*>(this);
+		if (index < container->numChildren ()) return container->findChildByIndex (index);
+		offset -= container->numChildren ();
+		if (isType (PackageEnv)) {
+			if (offset == 0 && slots_pseudo_object) return slots_pseudo_object;
+			return static_cast<const REnvironmentObject*>(this)->namespaceEnvironment ();
+		}
+	}
+	if (offset == 0) return slots_pseudo_object;
+	return 0;
+}
+
+RKEditor *RObject::editor () const {
+	return (RKGlobals::tracker ()->objectEditor (this));
 }
 
 void RObject::rename (const QString &new_short_name) {
 	RK_TRACE (OBJECTS);
-	parent->renameChild (this, new_short_name);
+	RK_ASSERT (canRename ());
+	static_cast<RContainerObject*> (parent)->renameChild (this, new_short_name);
 }
 
 void RObject::remove (bool removed_in_workspace) {
 	RK_TRACE (OBJECTS);
-	parent->removeChild (this, removed_in_workspace);
+	RK_ASSERT (canRemove ());
+
+	if (isSlotsPseudoObject ()) {
+		RK_ASSERT (removed_in_workspace);
+		parent->slots_pseudo_object = 0;
+		delete this;
+	} else if (isPackageNamespace ()) {
+		RK_ASSERT (removed_in_workspace);
+		RK_ASSERT (parent->isType (Environment));
+		static_cast<REnvironmentObject*> (parent)->namespace_envir = 0;
+	} else {
+		static_cast<RContainerObject*> (parent)->removeChild (this, removed_in_workspace);
+	}
 }
 
 //static
@@ -533,17 +595,82 @@ RObject::RDataType RObject::textToType (const QString &text) {
 
 //static
 QString RObject::rQuote (const QString &string) {
-	// TODO: this is not entirely correct, yet (let alone efficient)!
-	QString copy = string;
-	return ("\"" + copy.replace (QRegExp ("\""), "\\\"") + "\"");
+	return (RKRSharedFunctionality::quote (string));
 }
 
-// static
-QString RObject::canonifyName (const QString &from) {
+//static 
+QStringList RObject::parseObjectPath (const QString &path) {
 	RK_TRACE (OBJECTS);
 
-	QString copy = from;
-	return (copy.replace ("[\"", "$").replace ('[', "").replace ("\"]", "").replace (']', ""));
+	QStringList ret;
+	QString fragment;
+
+	int end = path.length ();
+	QChar quote_char;
+	bool escaped = false;
+	bool seek_bracket_end = false;
+	for (int i = 0; i < end; ++i) {
+		QChar c = path.at (i);
+		if (quote_char.isNull ()) {
+			if (c == '\'' || c == '\"' || c == '`') {
+				quote_char = c;
+			} else {
+				if (!seek_bracket_end) {
+					if (c == '$') {
+						ret.append (fragment);
+						ret.append ("$");
+						fragment.clear ();
+					} else if (c == '[') {
+						ret.append (fragment);
+						ret.append ("$");
+						fragment.clear ();
+						if ((i+1 < end) && (path.at (i+1) == '[')) ++i;
+						seek_bracket_end = true;
+					} else if (c == ':') {
+						ret.append (fragment);
+						if ((i+1 < end) && (path.at (i+1) == ':')) ++i;
+						if ((i+1 < end) && (path.at (i+1) == ':')) {
+							++i;
+							ret.append (":::");
+						} else ret.append ("::");
+						fragment.clear ();
+					} else if (c == '@') {
+						ret.append (fragment);
+						ret.append ("@");
+						fragment.clear ();
+					} else {
+						fragment.append (c);
+					}
+				} else {
+					if (c == ']') {
+						if ((i+1 < end) && (path.at (i+1) == ']')) ++i;
+						seek_bracket_end = false;
+						continue;
+					} else {
+						fragment.append (c);
+					}
+				}
+			}
+		} else {	// inside a quote
+			if (c == '\\') escaped = !escaped;
+			else {
+				if (escaped) {
+					if (c == 't') fragment.append ('\t');
+					else if (c == 'n') fragment.append ('\n');
+					else fragment.append ('\\' + c);
+				} else {
+					if (c == quote_char) {
+						quote_char = QChar ();
+					} else {
+						fragment.append (c);
+					}
+				}
+			}
+		}
+	}
+	if (!fragment.isEmpty ()) ret.append (fragment);
+	RK_DO (qDebug ("parsed object path %s into %s", qPrintable (path), qPrintable (ret.join ("-"))), OBJECTS, DL_DEBUG);
+	return ret;
 }
 
 //virtual
@@ -569,10 +696,12 @@ bool RObject::canRead () const {
 
 	return (this != RObjectList::getObjectList ());
 }
-
+ 
 bool RObject::canRename () const {
 	RK_TRACE (OBJECTS);
 
+	if (isPseudoObject ()) return false;
+	if (parent && parent->isSlotsPseudoObject ()) return false;
 	// TODO: find out, if binding is locked:
 	// if (isLocked ()) return false;
 	return (isInGlobalEnv ());
@@ -581,6 +710,8 @@ bool RObject::canRename () const {
 bool RObject::canRemove () const {
 	RK_TRACE (OBJECTS);
 
+	if (isPseudoObject ()) return false;
+	if (parent && parent->isSlotsPseudoObject ()) return false;
 	// TODO: find out, if binding is locked:
 	// if (isLocked ()) return false;
 	return (isInGlobalEnv ());
