@@ -2,7 +2,7 @@
                           robject  -  description
                              -------------------
     begin                : Thu Aug 19 2004
-    copyright            : (C) 2004, 2006, 2007, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -24,6 +24,7 @@
 
 #include "../rbackend/rcommandreceiver.h"
 
+class RSlotsPseudoObject;
 class RContainerObject;
 class RCommandChain;
 class RKEditor;
@@ -39,7 +40,7 @@ Base class for representations of objects in the R-workspace. RObject is never u
 
 class RObject : public RCommandReceiver {
 public:
-	RObject (RContainerObject *parent, const QString &name);
+	RObject (RObject *parent, const QString &name);
 	virtual ~RObject ();
 
 /** types of objects, RKWard knows about */
@@ -56,13 +57,16 @@ public:
 		GlobalEnv=1 << 9,
 		ToplevelEnv=1 << 10,
 		PackageEnv=1 << 11,
-		HasMetaObject=1 << 12,
-		Misplaced=1 << 13,		/** < the object is not in the namespace where it would be expected */
+		Misplaced=1 << 12,		/** < the object is not in the namespace where it would be expected */
+		S4Object=1 << 13,
 		Numeric=1 << 14,
 		Factor=2 << 14,
 		Character=3 << 14,
 		Logical=4 << 14,
 		DataTypeMask=Numeric | Factor | Character | Logical,
+		PseudoObject = 1 << 26, /** < The object is an internal representation, only, and does not exist in R. Currently, this is the case only for the slots-pseudo object */
+		Updating=1 << 27, /** < The object is about to be updated from R */
+		Incomplete=1 << 28,	/** < The information on this object is not complete (typically, it's children have not been scanned, yet). */
 		NonVisibleObject=1 << 29,	/** < the object is not listed in the object list. Currently, this is only the case for row.names()-objects */
 		NeedDataUpdate=1 << 30,	/** < the object's data should be (re-) fetched from R. The main purpose of this flag is to make sure the data is synced *after* the structure has been synced */
 		Pending=1 << 31		/** < the object is pending, i.e. it has been created in the object list, but we have not seen it in R, yet. This is used by data editors to create the illusion that a new object was added immediately, while in fact it takes some time to create it in the backend. */
@@ -79,7 +83,22 @@ public:
 		MaxKnownDataType = DataLogical
 	};
 
-	#define ROBJECT_TYPE_INTERNAL_MASK (RObject::Container | RObject::Variable | RObject::Workspace | RObject::Environment | RObject::Function)
+/** For passing data between RKStructureGetter and RObject. Be very careful about changing the values in this enum. It is for better readability / searchability of the code, only. */
+	enum {
+		StoragePositionName = 0,
+		StoragePositionType = 1,
+		StoragePositionClass = 2,
+		StoragePositionMeta = 3,
+		StoragePositionDims = 4,
+		StoragePositionSlots = 5,
+		StoragePositionChildren = 6,
+		StoragePositionNamespace = 7,
+		StoragePositionFunArgs = 6,
+		StoragePositionFunValues = 7,
+		StorageSizeBasicInfo = 6,
+	};
+
+#define ROBJECT_TYPE_INTERNAL_MASK (RObject::Container | RObject::Variable | RObject::Workspace | RObject::Environment | RObject::Function)
 /** @returns false if an object of the given old type cannot represent an object of the given new type (e.g. (new_type & RObjectType::Variable), but (old_type & RObjectType::Container)). */
 	static bool isMatchingType (int old_type, int new_type) { return ((old_type & ROBJECT_TYPE_INTERNAL_MASK) == (new_type & ROBJECT_TYPE_INTERNAL_MASK)); };
 	
@@ -90,15 +109,18 @@ public:
 	QString getMetaProperty (const QString &id) const;
 	QString getDescription () const;
 	
-	virtual void setLabel (const QString &value, bool sync=true);
-	virtual void setMetaProperty (const QString &id, const QString &value, bool sync=true);
+	void setLabel (const QString &value, bool sync=true);
+	void setMetaProperty (const QString &id, const QString &value, bool sync=true);
 	
 	bool isContainer () const { return (type & (Container | Environment | Workspace)); };
 	bool isDataFrame () const { return (type & DataFrame); };
 	bool isVariable () const { return (type & Variable); };
 	/** see RObjectType */
 	bool isType (int type) const { return (RObject::type & type); };
-	bool hasMetaObject () const { return (type & HasMetaObject); };
+	bool isPseudoObject () const { return isType (PseudoObject); };
+	bool isSlotsPseudoObject () const { return (this && parent && ((void*) parent->slots_pseudo_object == (void*) this)); };
+	bool isPackageNamespace () const { return (this && isPseudoObject () && (name == "NAMESPACE")); };
+	bool hasMetaObject () const { return (meta_map); };
 	/** see RObjectType::Pending */
 	bool isPending () const { return type & Pending; };
 
@@ -109,6 +131,8 @@ public:
 /** mark the data of this object and all of its children as dirty (recursively). Dirty data will be updated *after* the new structure update (if the object is opened for editing) */
 	void markDataDirty ();
 
+/** Returns the editor of this object, if any, or 0 */
+	RKEditor* editor () const;
 	bool canEdit () const;
 	bool canRead () const;
 	bool canRename () const;
@@ -118,20 +142,22 @@ public:
 	void rename (const QString &new_short_name);
 	void remove (bool removed_in_workspace);
 
-	unsigned int numClasses () const { return num_classes; };
-	QString getClassName (int index) const { return classnames[index]; };
+	const QStringList &classNames () const { return classnames; };
 	QString makeClassString (const QString &sep) const;
 /** @param class_name the name of the class to check for
 @returns true, if the object has (among others) the given class, false otherwise */
 	bool inherits (const QString &class_name) const;
 
-/** get number of dimensions. For simplicity, In RKWard each object is considered to have at least one dimension (but that dimension may be 0 in length) */
-	unsigned int numDimensions () const { return num_dimensions; };
-/** get the length of the given dimension. The object is guaranteed to have at least 1 dimension, so calling getDimension (0) is always safe */
-	int getDimension (int index) const { return dimensions[index]; };
+/** get vector of dimensions. For simplicity, In RKWard each object is considered to have at least one dimension (but that dimension may be 0 in length) */
+	const QVector<qint32> &getDimensions () const { return dimensions; };
 /** short hand for getDimension (0). Meaningful for one-dimensional objects */
 	int getLength () const { return dimensions[0]; };
 
+	/** return the index of the given child, or -1 if there is no such child */
+	int getObjectModelIndexOf (RObject *child) const;
+	int numChildrenForObjectModel () const;
+	RObject *findChildByObjectModelIndex (int) const;
+
 /** A QList of RObjects. Internally the same as RObjectMap, but can be considered "public" */
 	typedef QList<RObject*> ObjectList;
 	typedef QMap<QString, RObject*> RObjectSearchMap;
@@ -142,8 +168,8 @@ public:
 /** write the MetaData to the backend. Commands will be issued in the given chain */
 	virtual void writeMetaData (RCommandChain *chain);
 
-/** Returns the parent / container of this object. All objects have a parent except for the RObjectList (which returns 0) */
-	RContainerObject *getContainer () const { return (parent); };
+/** Returns the parent of this object. All objects have a parent except for the RObjectList (which returns 0) */
+	RObject *parentObject () const { return (parent); };
 
 	RDataType getDataType () const { return (typeToDataType (type)); };
 	int getType () const { return type; };
@@ -160,16 +186,22 @@ public:
 	static QString rQuote (const QString &string);
 /** Returns a pretty description of the object, and its most important properties. TODO should this be virtual or not? I suppose, it's a close call. For now, we do all work here with casts */
 	QString getObjectDescription () const;
-/** Returns a canonified name given a non-canoified name. Warning! This is not (necessarily) suitable for submission to
-R, only for internal lookup. For submission to R, always use RObject::getFullName (), as it will apply more complicated (and correct) rules depending on object type */
-	static QString canonifyName (const QString &from);
+/** Parses an object path (such as package::name[["a"]]$b@slot) into its components, returning them as a list (in this case 'package', '::' 'name', '$', 'a', '$', 'b', '@', 'slot'). */
+	static QStringList parseObjectPath (const QString &path);
 /** Tests whether the given name is "irregular", i.e. contains spaces, quotes, operators, or the like. @see RContainerObject::validizeName () */
 	static bool irregularShortName (const QString &name);
-/** Function for code completion: given the partial name, find all objects matching this partial name
+/** try to find the object as a child object of this object.
+@param name of the object (relative to this object)
+@returns a pointer to the object (if found) or 0 if not found */
+	RObject *findObject (const QString &name) { return findObjects (parseObjectPath (name), 0, "$"); };
+	/** Function for code completion: given the partial name, find all objects matching this partial name
 @param partial_name The partial name to look up
-@param current_list A pointer to a valid (but probably initially empty) RObjectMap. Matches will be added to this list
-@param name_is_canonified internal parameter. Set to true, if the name to match is already canonfied (else it will be canonified internally) */
-	virtual void findObjectsMatching (const QString &partial_name, RObjectSearchMap *current_list, bool name_is_canonified=false) const;
+@param current_list A pointer to a valid (but probably initially empty) RObjectMap. Matches will be added to this list */
+	void findObjectsMatching (const QString &partial_name, RObjectSearchMap *current_list) { findObjects (parseObjectPath (partial_name), current_list, "$"); };
+
+/** Fetch more levels of object representation (if needed). Note: Data is fetched asynchronously. 
+@param levels levels to recurse (0 = only direct children). */
+	void fetchMoreIfNeeded (int levels=1);
 
 /// For now, the ChangeSet only handles RKVariables!
 	struct ChangeSet {
@@ -177,11 +209,6 @@ R, only for internal lookup. For submission to R, always use RObject::getFullNam
 		int to_index;
 	};
 
-/** try to find the object as a child object of this object. Default implementation always returns 0, as this is not a container
-@param name of the object (relative to this object)
-@param is_canonified the object name may usually have to be canonified. Since this function may be called recursively, canonification may already have occurred on a higher level. In this case the argument is set to true to avoid some duplicate work. When calling from outside always leave the default false.
-@returns a pointer to the object (if found) or 0 if not found */
-	virtual RObject *findObject (const QString &name, bool is_canonified=false) const;
 /** generates a (full) name for a child of this object with the given name. */
 	virtual QString makeChildName (const QString &short_child_name, bool misplaced=false) const;
 protected:
@@ -192,13 +219,15 @@ protected:
 /** A map of objects accessible by index. Used in RContainerObject. Defined here for technical reasons. */
 	typedef QList<RObject*> RObjectMap;
 
-	RContainerObject *parent;
+	RObject *parent;
+	RSlotsPseudoObject *slots_pseudo_object;
 	QString name;
 	int type;
-	int *dimensions;
-	unsigned int num_dimensions;
-	QString *classnames;
-	unsigned int num_classes;
+	QVector<qint32> dimensions;
+	QStringList classnames;
+
+/** Worker function for findObject() and findObjectsMatching(). If matches != 0, look for partial matches, and store them in the map (findObjectsMatching()). Else look for exact matches and return the first match (findObject()). */
+	virtual RObject *findObjects (const QStringList &path, RObjectSearchMap *matches, const QString &op);
 
 	virtual QString makeChildBaseName (const QString &short_child_name) const;
 
@@ -233,6 +262,10 @@ protected:
 @param new_data The command. Make sure it really is the dims field of an .rk.get.structure-command to update classes *before* calling this function! WARNING: the new_data object may get changed during this call. Call canAccommodateStructure () before calling this function!
 @returns whether this caused any changes */
 	bool updateDimensions (RData *new_data);
+/** update information on slots of this object (if it is an S4 object)
+@param new_data The command. Make sure it really is the slots field of an .rk.get.structure-command to update classes *before* calling this function! WARNING: the new_data object may get changed during this call. Call canAccommodateStructure () before calling this function!
+@returns whether this caused any changes */
+	bool updateSlots (RData *new_data);
 
 friend class RKModificationTracker;
 /** Notify the object that some model needs its data. The object should take care of fetching the data from the backend, unless it already has the data. The default implementation does nothing (raises an assert). */
