@@ -2,7 +2,7 @@
                           rkvareditmodel  -  description
                              -------------------
     begin                : Mon Nov 05 2007
-    copyright            : (C) 2007, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2007, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -21,6 +21,7 @@
 
 #include <QColor>
 #include <QTimer>
+#include <QFont>
 
 #include "../core/rcontainerobject.h"
 #include "../core/rkmodificationtracker.h"
@@ -40,6 +41,7 @@ RKVarEditModel::RKVarEditModel (QObject *parent) : RKVarEditModelBase (parent),
 	rownames = 0;
 	header_locked = false;
 	duplicate_check_triggered = false;
+	reset_scheduled = false;
 
 	addNotificationType (RObjectListener::ObjectRemoved);
 	addNotificationType (RObjectListener::MetaChanged);
@@ -70,7 +72,7 @@ void RKVarEditModel::addObject (int index, RKVariable* object) {
 
 	beginInsertColumns (QModelIndex (), index, index);
 	if (meta_model) meta_model->beginAddDataObject (index);
-	if (object->isPending ()) object->setLength (trueRows ());	// probably we just created it ourselves
+	if (object->isPending () && (!object->getLength ())) object->setLength (trueRows ());	// probably we just created it ourselves
 	listenForObject (object);
 	objects.insert (index, object);
 	if (meta_model) meta_model->endAddDataObject ();
@@ -133,8 +135,27 @@ void RKVarEditModel::objectMetaChanged (RObject* changed) {
 	int cindex = objects.indexOf (static_cast<RKVariable*> (changed));	// no check for isVariable needed. we only need to look up, if we have this object, and where.
 	if (cindex < 0) return;	// none of our buisiness
 
-	emit (dataChanged (index (0, cindex), index (trueRows (), cindex)));
 	if (meta_model) meta_model->objectMetaChanged (cindex);
+
+	if (!reset_scheduled) {
+		reset_scheduled = true;
+		QTimer::singleShot (0, this, SLOT (doResetNow()));
+#if QT_VERSION >= 0x040600
+		beginResetModel ();
+#endif
+	}
+}
+
+void RKVarEditModel::doResetNow () {
+	RK_TRACE (EDITOR);
+	RK_ASSERT (reset_scheduled);
+
+	reset_scheduled = false;
+#if QT_VERSION >= 0x040600
+	endResetModel ();
+#else
+	reset ();
+#endif
 }
 
 void RKVarEditModel::objectDataChanged (RObject* object, const RObject::ChangeSet *changes) {
@@ -179,7 +200,6 @@ bool RKVarEditModel::insertRows (int row, int count, const QModelIndex& parent)
 // TODO: this does not emit any data change notifications to other editors
 		objects[i]->insertRows (row, count);
 	}
-#warning TODO: adjust dimensions of the data.frame!
 	endInsertRows ();
 
 	return true;
@@ -345,7 +365,8 @@ QVariant RKVarEditModel::headerData (int section, Qt::Orientation orientation, i
 
 	if (orientation == Qt::Horizontal) {
 		if (section >= objects.size ()) return i18n ("#New Variable#");
-		return objects[section]->getShortName ();
+		if (section < var_col_offset) return i18n ("Row names");
+		return (QString::number (section - var_col_offset + 1));
 	}
 
 	if (section < rownames->getLength ()) {
@@ -535,6 +556,12 @@ QVariant RKVarEditMetaModel::data (const QModelIndex& index, int role) const {
 		if (row == LevelsRow) return var->getValueLabelString ();
 	}
 
+	if ((role == Qt::FontRole) && (row == NameRow)) {
+		QFont font;
+		font.setBold (true);
+		return (font);
+	}
+
 	return QVariant ();
 }
 
@@ -590,7 +617,10 @@ bool RKVarEditMetaModel::setData (const QModelIndex& index, const QVariant& valu
 
 	if (row == NameRow) {
 		if (var->getShortName () != value.toString ()) {
-			RKGlobals::tracker ()->renameObject (var, var->getContainer ()->validizeName (value.toString ()));
+			if (!var->canRename ()) return false;
+			if (var->parentObject ()->isContainer ()) {
+				RKGlobals::tracker ()->renameObject (var, static_cast<RContainerObject*> (var->parentObject ())->validizeName (value.toString ()));
+			} else return false;
 		}
 	} else if (row == LabelRow) {
 		var->setLabel (value.toString (), true);
@@ -643,6 +673,13 @@ QVariant RKVarEditMetaModel::headerData (int section, Qt::Orientation orientatio
 		if (section == FormatRow) return (i18n ("Format"));
 		if (section == LevelsRow) return (i18n ("Levels"));
 	}
+	if (role == Qt::ToolTipRole) {
+		if (section == NameRow) return (i18n ("Edit these fields to rename variables."));
+		if (section == LabelRow) return (i18n ("A descriptive label for each column (optional)."));
+		if (section == TypeRow) return (i18n ("Type of data."));
+		if (section == FormatRow) return (i18n ("Double click on these fields to customize data display."));
+		if (section == LevelsRow) return (i18n ("Double click on these fields to edit factor levels."));
+	}
 
 	return QVariant ();
 }
@@ -756,6 +793,8 @@ RKVarEditDataFrameModel::RKVarEditDataFrameModel (const QString& validized_name,
 	for (int i = 0; i < initial_cols; ++i) {
 		RObject* child = df->createPendingChild (df->validizeName (QString ()), -1, false, false);
 		RK_ASSERT (child->isVariable ());
+		// let's start with one (empty) row, to avoid confusion
+		static_cast<RKVariable*> (child)->setLength (1);
 	}
 
 	init (df);
@@ -804,11 +843,11 @@ bool RKVarEditDataFrameModel::insertColumns (int column, int count, const QModel
 	if (column > trueCols ()) column = trueCols ();
 	if (column < var_col_offset) column = var_col_offset;
 	for (int col = column; col < (column + count); ++col) {
-		RObject *obj = dataframe->createPendingChild (dataframe->validizeName (QString ()), col);
+		RObject *obj = dataframe->createPendingChild (dataframe->validizeName (QString ()), col-var_col_offset);
 		RK_ASSERT (obj->isVariable ());
 //		addObject (col, obj);	// the object will be added via RKModificationTracker::addObject -> this::childAdded. That will also take care of calling beginInsertColumns()/endInsertColumns()
 	
-		RKGlobals::rInterface ()->issueCommand (new RCommand (".rk.data.frame.insert.column (" + dataframe->getFullName () + ", \"" + obj->getShortName () + "\", " + QString::number (col+1) + ")", RCommand::App | RCommand::Sync));
+		RKGlobals::rInterface ()->issueCommand (new RCommand (".rk.data.frame.insert.column (" + dataframe->getFullName () + ", \"" + obj->getShortName () + "\", " + QString::number (col+1-var_col_offset) + ")", RCommand::App | RCommand::Sync));
 	}
 
 	return true;
