@@ -2,7 +2,7 @@
                           main.cpp  -  description
                              -------------------
     begin                : Tue Oct 29 20:06:08 CET 2002
-    copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010 by Thomas Friedrichsmeier 
+    copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by Thomas Friedrichsmeier 
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -54,8 +54,10 @@
 #include <kaboutdata.h>
 #include <klocale.h>
 #include <ktemporaryfile.h>
+#include <kstandarddirs.h>
 
 #include <qstring.h>
+#include <QMutex>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -66,7 +68,6 @@
 
 #ifdef Q_WS_WIN
 	// these are needed for the exit hack.
-#	include <stdio.h>
 #	include <windows.h>
 #endif
 
@@ -77,27 +78,36 @@
 int RK_Debug_Level = 0;
 int RK_Debug_Flags = ALL;
 int RK_Debug_CommandStep = 0;
+QMutex RK_Debug_Mutex;
 
 static KCmdLineOptions options;
 
 void RKDebugMessageOutput (QtMsgType type, const char *msg) {
+	RK_Debug_Mutex.lock ();
 	if (type == QtFatalMsg) {
 		fprintf (stderr, "%s\n", msg);
 	}
 	RKSettingsModuleDebug::debug_file->write (msg);
 	RKSettingsModuleDebug::debug_file->write ("\n");
 	RKSettingsModuleDebug::debug_file->flush ();
+	RK_Debug_Mutex.unlock ();
 }
 
 int main(int argc, char *argv[]) {
+/* #ifdef Q_WS_X11
+	This (along with the proper includes, of course) makes library (gWidgetsRGtk2) work on X11 on some systems.
+	Unfortunately, on others, I causes lockups when plotting.
+
+	XInitThreads ();
+	XtToolkitThreadInitialize ();
+#endif */
 	options.add ("evaluate <Rcode>", ki18n ("After starting (and after loading the specified workspace, if applicable), evaluate the given R code."), 0);
 	options.add ("debug-level <level>", ki18n ("Verbosity of debug messages (0-5)"), "2");
 	options.add ("debug-flags <flags>", ki18n ("Mask for components to debug (see debug.h)"), "8191");
 	options.add ("debugger <command>", ki18n ("Debugger (enclose any debugger arguments in single quotes ('') together with the command)"), "");
-	options.add ("disable-stack-check", ki18n ("Disable R C stack checking"), 0);
 	options.add ("+[File]", ki18n ("R workspace file to open"), 0);
 
-	KAboutData aboutData("rkward", QByteArray (), ki18n ("RKWard"), VERSION, ki18n ("Frontend to the R statistics language"), KAboutData::License_GPL, ki18n ("(c) 2002, 2004, 2005, 2006, 2007, 2008, 2009, 2010"), KLocalizedString (), "http://rkward.sf.net", "rkward-devel@lists.sourceforge.net");
+	KAboutData aboutData("rkward", QByteArray (), ki18n ("RKWard"), RKWARD_VERSION, ki18n ("Frontend to the R statistics language"), KAboutData::License_GPL, ki18n ("(c) 2002, 2004, 2005, 2006, 2007, 2008, 2009, 2010"), KLocalizedString (), "http://rkward.sf.net", "rkward-devel@lists.sourceforge.net");
 	aboutData.addAuthor (ki18n ("%1").subs ("Thomas Friedrichsmeier"), ki18n ("Project leader / main developer"));
 	aboutData.addAuthor (ki18n ("%1").subs ("Pierre Ecochard"), ki18n ("C++ coder since 0.2.9"));
 	aboutData.addAuthor (ki18n ("%1").subs ("Stefan Roediger"), ki18n ("Many plugins, suggestions, marketing, translations"));
@@ -122,7 +132,7 @@ int main(int argc, char *argv[]) {
 	// This is so it prints "Usage rkward..." instead of "Usage rkward.bin...", etc.
 	// it seems safest to keep a copy, since the shell still owns argv
 	char *argv_copy[argc];
-	argv_copy[0] = qstrdup (QString (argv[0]).remove (".bin").replace (".exe", ".bat").toLocal8Bit ());
+	argv_copy[0] = qstrdup (QString (argv[0]).remove (".frontend").replace (".exe", ".bat").toLocal8Bit ());
 	for (int i = 1; i < argc; ++i) {
 		argv_copy[i] = argv[i];
 	}
@@ -140,14 +150,16 @@ int main(int argc, char *argv[]) {
 	if (args->count ()) {
 		stoptions->initial_url = KUrl (args->makeURL (args->arg (0).toLatin1()));
 	}
-	stoptions->no_stack_check = args->isSet ("disable-stack-check");
 	stoptions->evaluate = args->getOption ("evaluate");
 
 	RKWardApplication app;
 	// install message handler *after* the componentData has been initialized
 	RKSettingsModuleDebug::debug_file = new KTemporaryFile ();
 	RKSettingsModuleDebug::debug_file->setAutoRemove (false);
-	if (RKSettingsModuleDebug::debug_file->open ()) qInstallMsgHandler (RKDebugMessageOutput);
+	if (RKSettingsModuleDebug::debug_file->open ()) {
+		RK_DO (qDebug ("Full debug output is at %s", qPrintable (RKSettingsModuleDebug::debug_file->fileName ())), APP, DL_INFO);
+		qInstallMsgHandler (RKDebugMessageOutput);
+	}
 
 	if (app.isSessionRestored ()) {
 		RESTORE(RKWardMainWindow);	// well, whatever this is supposed to do -> TODO
@@ -156,11 +168,15 @@ int main(int argc, char *argv[]) {
 	}
 	args->clear();
 
+	// Usually, KDE always adds the current directory to the list of prefixes.
+	// However, since RKWard 0.5.6, the main binary is in KDE's libexec dir, which defies this mechanism. Therefore, RKWARD_ENSURE_PREFIX is set from the wrapper script.
+	char *add_path = getenv ("RKWARD_ENSURE_PREFIX");
+	if (add_path) KGlobal::dirs ()->addPrefix (QString::fromLocal8Bit (add_path));
+
 	// do it!
 	int status = app.exec ();
 
 	qInstallMsgHandler (0);
-	RK_DO (qDebug ("Full debug output is at %s", qPrintable (RKSettingsModuleDebug::debug_file->fileName ())), APP, DL_INFO);
 	RKSettingsModuleDebug::debug_file->close ();
 
 #ifdef Q_WS_WIN
