@@ -1,7 +1,7 @@
 /***************************************************************************
                           multistringselector  -  description
                              -------------------
-    begin                : Fir Sep 10 2005
+    begin                : Fri Sep 10 2005
     copyright            : (C) 2005 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
@@ -22,7 +22,8 @@
 #include <qstringlist.h>
 #include <qstring.h>
 
-class QListView;
+class QTreeWidget;
+class QTreeWidgetItem;
 class QPushButton;
 
 /** This convenience widget allows to select one or more strings (e.g. filenames) and sort them in any order. The function to acutally select new strings to add to the selection is not implemented in this class for more flexibility. Rather, connect to the getNewStrings () signal and assign the desired QString(s) in a custom slot.
@@ -47,7 +48,10 @@ public slots:
 	void downButtonClicked ();
 	void listSelectionChanged ();
 private:
-	QListView* list_view;
+/** convenience function to get the (single) selected item of the tree_view, or 0 if no item is selected */
+	QTreeWidgetItem* treeSelectedItem () const;
+
+	QTreeWidget* tree_view;
 	QPushButton* add_button;
 	QPushButton* remove_button;
 	QPushButton* up_button;
