@@ -21,6 +21,7 @@
 #include <qregexp.h>
 #include <QDir>
 
+#include <klocale.h>
 #include <kxmlguiclient.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
@@ -171,6 +172,7 @@ namespace RKCommonFunctions {
 			QChar c = in[i];
 			if (c == '\\') out.append ("\\\\");
 			else if (c == '\n') out.append ("\\n");
+			else if (c == '\t') out.append ("\\t");
 			else if (c == '"') out.append ("\\\"");
 			else out.append (c);
 		}
@@ -188,6 +190,7 @@ namespace RKCommonFunctions {
 				if (i >= in.size ()) break;
 				c = in[i];
 				if (c == 'n') c = '\n';
+				if (c == 't') c = '\t';
 				else if (c == '"') c = '"';
 			}
 			out.append (c);
@@ -195,4 +198,21 @@ namespace RKCommonFunctions {
 
 		return out;
 	}
+
+	void setTips (const QString tip, QWidget *first, QWidget *second, QWidget *third) {
+		for (int i=0; i < 3; ++i) {
+			QWidget *w = first;
+			if (i == 1) w = second;
+			if (i == 2) w = third;
+			if (!w) return;
+
+			w->setToolTip (tip);
+			w->setWhatsThis (tip);
+		}
+	}
+
+	QString noteSettingsTakesEffectAfterRestart () {
+		return (i18n ("<p><em>Note:</em> This setting does not take effect until you restart RKWard.</p>"));
+	}
+
 }	// namespace
