@@ -2,7 +2,7 @@
                           rkcomponent  -  description
                              -------------------
     begin                : Tue Dec 13 2005
-    copyright            : (C) 2005, 2006, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -58,7 +58,7 @@ void RKComponentBase::fetchPropertyValuesRecursive (QMap<QString, QString> *list
 			}
 		} else {
 			RK_ASSERT (it.value ()->isComponent ());
-			if (!static_cast<RKComponent *> (it.value ())->isEnabled ()) continue;
+			if (static_cast<RKComponent *> (it.value ())->isInactive ()) continue;
 			it.value ()->fetchPropertyValuesRecursive (list, true, prefix + it.key () + '.');
 		}
 	}
@@ -145,19 +145,12 @@ QString RKComponentBase::value (const QString &modifier) {
 	return QString ();
 }
 
-bool RKComponentBase::isSatisfied () {
-	RK_TRACE (PLUGIN);
-	if (!required) return true;
-	if (isValid ()) return true;
- 	if (isComponent ()) RK_DO (qDebug ("component not satisfied: %s", qPrintable (static_cast<RKComponent*> (this)->getIdInParent ())), PLUGIN, DL_DEBUG);
-	return false;		// never happens in RKComponentBase, but might in subclasses
-}
-
 RKComponentBase::ComponentStatus RKComponentBase::recursiveStatus () {
 	RK_TRACE (PLUGIN);
 
 	bool processing = false;
 	bool children_satisfied = true;
+	// we always need to interate over all children, since we need to make sure to find any which are dead or processing.
 	for (QHash<QString, RKComponentBase*>::const_iterator it = child_map.constBegin (); it != child_map.constEnd (); ++it) {
 		ComponentStatus s = it.value ()->recursiveStatus ();
 		if (s == Dead) return Dead;
@@ -165,8 +158,11 @@ RKComponentBase::ComponentStatus RKComponentBase::recursiveStatus () {
 		else if (s != Satisfied) children_satisfied = false;
 	}
 	if (processing) return Processing;
-	if (!required) return Satisfied;
-	if (children_satisfied && isSatisfied ()) return Satisfied;
+	bool req = required;
+	if (isComponent () && static_cast<RKComponent*>(this)->isInactive ()) req = false;
+	if (!req) return Satisfied;
+	if (children_satisfied && isValid ()) return Satisfied;
+ 	if (isComponent ()) RK_DO (qDebug ("component not satisfied: %s", qPrintable (static_cast<RKComponent*> (this)->getIdInParent ())), PLUGIN, DL_DEBUG);
 	return Unsatisfied;
 }
 
@@ -243,14 +239,11 @@ void RKComponent::updateEnablednessRecursive () {
 	}
 }
 
-bool RKComponent::isValid () {
-	RK_TRACE (PLUGIN);
-#warning TODO: I do not think we need this. Use recursiveStatus, instead
-
-	for (QHash<QString, RKComponentBase*>::const_iterator it = child_map.constBegin (); it != child_map.constEnd (); ++it) {
-		if (!(it.value ()->isSatisfied ())) return false;
-	}
-	return true;
+bool RKComponent::isInactive () {
+	if (!isEnabled ()) return true;
+	if (parentWidget () && isHidden ()) return true;	// Note: Components embedded as button may be "hidden" without being inaccessible
+	if (!visibility_property->boolValue ()) return true;	// Note for those, this is the appropriate check
+	return false;
 }
 
 bool RKComponent::isWizardish () {
