@@ -58,7 +58,7 @@ RKPluginSpinBox::RKPluginSpinBox (const QDomElement &element, RKComponent *paren
 
 		realvalue->setMin (min);
 		realvalue->setMax (max);
-		realvalue->setPrecision (max_precision);
+		realvalue->setPrecision (default_precision);
 	} else {
 		int min = xml->getIntAttribute (element, "min", INT_MIN, DL_INFO);
 		int max = xml->getIntAttribute (element, "max", INT_MAX, DL_INFO);
