@@ -144,6 +144,11 @@ void RKStandardComponentGUI::showEvent (QShowEvent *e) {
 
 	QWidget::showEvent (e);
 
+	// HACK: Workaround for this issue (caused by a mysterious Qt bug, apparently): http://www.mail-archive.com/rkward-devel@lists.sourceforge.net/msg01340.html
+	QSize min = minimumSize ();
+	if ((min.width () < 50) || (min.height () < 50)) min = sizeHint ();
+	setMinimumSize (min.expandedTo (QSize (50, 50)));
+
 	if (toggle_code_button) {	// this is a dialog, not  wizard
 		QTimer::singleShot (0, this, SLOT (toggleCode ()));
 	}
