@@ -47,7 +47,7 @@ function calculate () {
 
 function printout () {
 	// check whether parameter estimations should be kept in the global enviroment
-	var save         = getValue("chk_save");
+	var save         = getValue("save_name.active");
 	var save_name    = getValue("save_name");
 
 	echo ('rk.header ("GRM parameter estimation")\n');
@@ -59,6 +59,6 @@ function printout () {
 	// check if results are to be saved:
 	if (save && save_name) {
 		echo ('# keep results in current workspace\n');
-		echo (save_name + ' <<- estimates.grm\n');
+		echo ('.GlobalEnv$' + save_name + ' <- estimates.grm\n');
 	}
 }
