@@ -30,6 +30,15 @@ function calculate () {
 }
 
 function printout () {
+  	var rad_pvalue      = getValue("rad_pvalue");
+
 	echo ('rk.header ("Item-fit statistics (' + getValue("x") + ')")\n');
-	echo ('rk.print (itemfit.res)\n');
+	echo ('rk.print ("Call:")\n');
+	echo ('rk.print.literal (deparse(itemfit.res$call, width.cutoff=500))\n');
+	echo ('rk.print ("Alternative: Items do not fit the model")\n');
+	echo ('rk.print (paste("Ability Categories:", itemfit.res$G))\n');
+	if (rad_pvalue == "montecarlo")
+	    echo ('rk.print (paste("Monte Carlo samples:", itemfit.res$B))\n');
+	echo ('rk.header ("Item-Fit Statistics and P-values:", level=4)\n');
+	echo ('rk.print(cbind("X^2"=round(itemfit.res$Tobs, digits=3), "Pr (&gt;X^2)"=format(round(itemfit.res$p.values, digits=3), nsmall=3)))\n');
 }
