@@ -3,17 +3,14 @@ function preprocess () {
 }
 
 function calculate () {
-	var vars = "substitute (" + trim (getValue ("x")).replace (/\n/g, "), substitute (") + ")";
-	var narm = "";
+	var narm = ", na.rm=FALSE";
 	if (getValue ("narm")) narm = ", na.rm=TRUE";
-	else narm = ", na.rm=FALSE";
+	var vars = trim (getValue ("x"));
 
-        echo ('objects <- list (' + vars + ')\n');
-	echo ('results <- data.frame (\'Variable Name\'=rep (NA, length (objects)), check.names=FALSE)\n');
-	echo ('for (i in 1:length (objects)) {\n');
-	echo ('	var <- eval (objects[[i]], envir=globalenv ())\n');
-	echo ('	results[i, \'Variable Name\'] <- rk.get.description (objects[[i]], is.substitute=TRUE)\n');
-	echo ('\n');
+	echo ('vars <- rk.list (' + vars.split ("\n").join (", ") + ')\n');
+	echo ('results <- data.frame (\'Variable Name\'=I(names (vars)), check.names=FALSE)\n');
+	echo ('for (i in 1:length (vars)) {\n');
+	echo ('	var <- vars[[i]]\n');
 	echo ('	try ({\n');
 	if (getValue ("skewness")) {
 		echo ('		results[i, \'Skewness\'] <- skewness (var' + narm + ')\n');
