@@ -1,10 +1,15 @@
 // globals
 var options;
 
-include ("plot_dist_common.php");
+include ('plot_dist_common.js');
 
 function getParameters () {
- 	if (getValue ("param") == "pprob") {
+	var size = "";
+	var size_label = "";
+	var paramTag = "";
+	var paramVal = "";
+	var paramLabel = "";
+	if (getValue ("param") == "pprob") {
 		size = getValue ("size_trial");
 		size_label = "Target for number of successful trials";
 		paramTag = ", prob=";
