@@ -8,13 +8,15 @@ function preview () {
 
 function doPrintout (full) {
 	var vars = getValue ("x");
-	var tabulate = getValue ("tabulate");
-	var tabulate_header = "";
-	if (tabulate) {
-		tabulate_header = '"Tabulate", "Yes"';
-	} else {
-		tabulate_header = '"Tabulate", "No"';
-	}
+
+	var tabulate = getValue ("tabulate.checked");
+	var main_header = '"Variable"=rk.get.description (' + vars + ')';
+	if (tabulate) main_header = getValue ('tabulate_options.parameters');
+
+	var limit = getValue ("limit.checked");
+	var limit_header = "";
+	if (limit) limit_header = ", " + getValue ('limit_options.parameters');
+
 	var radius = getValue ("radius");
 	var angle = getValue ("angle");
 	var angle_inc = getValue ("angle_inc");
@@ -24,9 +26,9 @@ function doPrintout (full) {
 	var clockwise = getValue ("clockwise");
 	var clockwise_header = "";
 	if (clockwise) {
-		clockwise_header = '"Clockwise", "Yes"';
+		clockwise_header = ', "Orientation"="Clockwise"';
 	} else {
-		clockwise_header = '"Clockwise", "No"';
+		clockwise_header = ', "Orientation"="Counter clockwise"';
 	}
 	var names_mode = getValue ("names_mode");
 
@@ -46,17 +48,21 @@ function doPrintout (full) {
 	var plotpost = getValue ("plotoptions.code.calculate");
 
 	if (tabulate) {
-		echo ('x <- table (' + vars + ', exclude=NULL)\n');
+		echo (getValue ('tabulate_options.code.calculate'));
 	} else {
-		echo ('x <- ' + vars);
-		echo ('\n');
+		echo ('x <- ' + getValue ("x") + '\n');
 		echo ('if (!is.numeric (x)) {\n');
-		echo ('	warning ("Data may not be numeric, but proceeding as requested.\\nDid you forget to check the tabulate option?")\n');
+		echo ('	warning ("Data is not numeric, but proceeding as requested.\\nDid you forget to check the tabulate option?")\n');
 		echo ('}\n');
 	}
+
+	if (getValue ("limit.checked")) {
+		echo (getValue ('limit_options.code.calculate'));
+	}
 	echo ('\n');
+
 	if (full) {
-		echo ('rk.header ("Pie chart", parameters=list ("Variable", rk.get.description (' + vars + '), ' + tabulate_header + ', ' + clockwise_header + '))\n');
+		echo ('rk.header ("Pie chart", parameters=list (' + main_header + limit_header + clockwise_header + '))\n');
 		echo ('\n');
 		echo ('rk.graph.on ()\n');
 	}
