@@ -25,6 +25,7 @@
 #include "../settings/rksettingsmodulegeneral.h"
 #include "../settings/rksettingsmoduleoutput.h"
 #include "../settings/rksettingsmodulegraphics.h"
+#include "../settings/rksettingsmoduleplugins.h"
 #include "../settings/rksettingsmoduledebug.h"
 #include "../core/robjectlist.h"
 #include "../core/renvironmentobject.h"
@@ -34,12 +35,14 @@
 #include "../dialogs/rkreadlinedialog.h"
 #include "../agents/showedittextfileagent.h"
 #include "../agents/rkeditobjectagent.h"
+#include "../agents/rkprintagent.h"
 #include "../windows/rcontrolwindow.h"
 #include "../windows/rkworkplace.h"
 #include "../windows/rkcommandlog.h"
 #include "../windows/rkhtmlwindow.h"
 #include "../plugin/rkcomponentmap.h"
 #include "../misc/rkcommonfunctions.h"
+#include "rksessionvars.h"
 
 #include "../windows/rkwindowcatcher.h"
 
@@ -59,12 +62,11 @@ RKWindowCatcher *window_catcher;
 #include <kstandarddirs.h>
 
 #include <qdir.h>
-#include <qtimer.h>
 #include <qvalidator.h>
 
 #include <stdlib.h>
 
-// update output (for immediate output commands) at least this often (msecs):
+// flush new pieces of output after this period of time:
 #define FLUSH_INTERVAL 100
 
 #define GET_LIB_PATHS 1
@@ -97,17 +99,19 @@ RInterface::RInterface () {
 	backend_dead = false;
 	num_active_output_record_requests = 0;
 	previous_output_type = ROutput::NoOutput;
+	flush_timer_id = 0;
 
 	// create a fake init command
 	RCommand *fake = new RCommand (i18n ("R Startup"), RCommand::App | RCommand::Sync | RCommand::ObjectListUpdate, i18n ("R Startup"), this, STARTUP_PHASE2_COMPLETE);
 	issueCommand (fake);
 
-	flush_timer = new QTimer (this);
-	connect (flush_timer, SIGNAL (timeout ()), this, SLOT (flushOutput ()));
-	flush_timer->start (FLUSH_INTERVAL);
-
+	new RKSessionVars (this);
 	new RKRBackendProtocolFrontend (this);
 	RKRBackendProtocolFrontend::instance ()->setupBackend ();
+
+	/////// Further initialization commands, which do not necessarily have to run before everything else can be queued, here. ///////
+	// NOTE: will receive the list as a call plain generic request from the backend ("updateInstalledPackagesList")
+	issueCommand (".rk.get.installed.packages()", RCommand::App | RCommand::Sync);
 }
 
 void RInterface::issueCommand (const QString &command, int type, const QString &rk_equiv, RCommandReceiver *receiver, int flags, RCommandChain *chain) {
@@ -119,7 +123,6 @@ RInterface::~RInterface(){
 	RK_TRACE (RBACKEND);
 
 	if (num_active_output_record_requests) RK_DO (qDebug ("%d requests for recording output still active on interface shutdown", num_active_output_record_requests), RBACKEND, DL_WARNING);
-	delete flush_timer;
 	delete window_catcher;
 }
 
@@ -232,7 +235,11 @@ void RInterface::doNextCommand (RCommand *command) {
 		RKCommandLog::getLog ()->addInput (command);
 
 		if (command_logfile_mode != NotRecordingCommands) {
-			if ((!(command->type () & RCommand::Sync)) || command_logfile_mode == RecordingCommandsWithSync) {
+			bool record = true;
+			if (command_logfile_mode != RecordingCommandsUnfiltered) {
+				if (command->type () & (RCommand::Silent | RCommand::Sync)) record = false;
+			}
+			if (record) {
 				command_logfile.write (command->command ().toUtf8 ());
 				command_logfile.write ("\n");
 			}
@@ -300,6 +307,13 @@ void RInterface::rCommandDone (RCommand *command) {
 void RInterface::handleRequest (RBackendRequest* request) {
 	RK_TRACE (RBACKEND);
 
+	if (request->type == RBackendRequest::OutputStartedNotification) {
+		RK_ASSERT (flush_timer_id == 0);
+		flush_timer_id = startTimer (FLUSH_INTERVAL);	// calls flushOutput (false); see timerEvent ()
+		RKRBackendProtocolFrontend::setRequestCompleted (request);
+		return;
+	}
+
 	flushOutput (true);
 	if (request->type == RBackendRequest::CommandOut) {
 		RCommandProxy *cproxy = request->takeCommand ();
@@ -342,19 +356,24 @@ void RInterface::handleRequest (RBackendRequest* request) {
 	}
 }
 
-void RInterface::flushOutput () {
+void RInterface::timerEvent (QTimerEvent *) {
 // do not trace. called periodically
-//	RK_TRACE (RBACKEND);
-
 	flushOutput (false);
 }
 
 void RInterface::flushOutput (bool forced) {
 // do not trace. called periodically
 //	RK_TRACE (RBACKEND);
-
 	ROutputList list = RKRBackendProtocolFrontend::instance ()->flushOutput (forced);
 
+	// this must come _after_ the output has been flushed.
+	if (forced || !list.isEmpty ()) {
+		if (flush_timer_id != 0) {
+			killTimer (flush_timer_id);
+			flush_timer_id = 0;
+		}
+	}
+
 	foreach (ROutput *output, list) {
 		if (all_current_commands.isEmpty ()) {
 			RK_DO (qDebug ("output without receiver'%s'", qPrintable (output->output)), RBACKEND, DL_WARNING);
@@ -497,6 +516,12 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 	} else if (call == "listPlugins") {
 		RK_ASSERT (calllist.count () == 1);
 		return RKComponentMap::getMap ()->allComponentIds ();
+	} else if (call == "loadPluginMaps") {
+		bool force = (calllist.value (1) == "force");
+		bool reload = (calllist.value (2) == "reload");
+		RKSettingsModulePlugins::registerPluginMaps (calllist.mid (3), force, reload);
+	} else if (call == "updateInstalledPackagesList") {
+		RKSessionVars::instance ()->setInstalledPackages (calllist.mid (1));
 	} else if (call == "showHTML") {
 		RK_ASSERT (calllist.count () == 2);
 		RKWorkplace::mainWorkplace ()->openHelpWindow (calllist.value (1));
@@ -546,7 +571,7 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 	} else if (call == "recordCommands") {
 		RK_ASSERT (calllist.count () == 3);
 		QString filename = calllist.value (1);
-		bool with_sync = (calllist.value (2) == "include.sync");
+		bool unfiltered = (calllist.value (2) == "include.all");
 
 		if (filename.isEmpty ()) {
 			command_logfile_mode = NotRecordingCommands;
@@ -558,8 +583,8 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 				command_logfile.setFileName (filename);
 				bool ok = command_logfile.open (QIODevice::WriteOnly | QIODevice::Truncate);
 				if (ok) {
-					command_logfile_mode = RecordingCommands;
-					if (with_sync) command_logfile_mode = RecordingCommandsWithSync;
+					if (unfiltered) command_logfile_mode = RecordingCommandsUnfiltered;
+					else command_logfile_mode = RecordingCommands;
 				} else {
 					return (QStringList ("Could not open file for writing. Not recording commands"));
 				}
@@ -579,6 +604,8 @@ QStringList RInterface::processPlainGenericRequest (const QStringList &calllist)
 		} else {
 			++num_active_output_record_requests;
 		}
+	} else if (call == "printPreview") {
+		RKPrintAgent::printPostscript (calllist.value (1), true);
 	} else {
 		return (QStringList ("Error: unrecognized request '" + call + "'."));
 	}
