@@ -61,7 +61,6 @@ void RKFrontendTransmitter::run () {
 	args.append ("--debug-level " + QString::number (RK_Debug_Level));
 	args.append ("--server-name " + server->fullServerName ());
 	args.append ("--data-dir " + RKSettingsModuleGeneral::filesPath ());
-	backend->setProcessChannelMode (QProcess::MergedChannels);	// at least for now. Seems difficult to get interleaving right, without this.
 	connect (backend, SIGNAL (finished (int, QProcess::ExitStatus)), this, SLOT (backendExit (int)));
 	QString backend_executable = KStandardDirs::findExe (QDir::toNativeSeparators (QCoreApplication::applicationDirPath () + "/rkward.rbackend"));
 	if (backend_executable.isEmpty ()) backend_executable = KStandardDirs::findExe (QDir::toNativeSeparators (QCoreApplication::applicationDirPath () + "/rbackend/rkward.rbackend"));	// for running directly from the build-dir
@@ -70,14 +69,9 @@ void RKFrontendTransmitter::run () {
 
 	// fetch security token
 	if (!backend->canReadLine ()) backend->waitForReadyRead ();
-	token = QString::fromLocal8Bit (backend->readLine ());
-	token.chop (1);
-#ifdef Q_WS_WIN
-	token.chop (1);	// '\r', I suppose...
-#endif
-
-	connect (backend, SIGNAL (readyReadStandardOutput ()), this, SLOT (newProcessOutput ()));
-	if (backend->bytesAvailable ()) newProcessOutput ();
+	token = QString::fromLocal8Bit (backend->readLine ()).trimmed ();
+	backend->closeReadChannel (QProcess::StandardError);
+	backend->closeReadChannel (QProcess::StandardOutput);
 
 	exec ();
 
@@ -110,13 +104,6 @@ void RKFrontendTransmitter::connectAndEnterLoop () {
 	setConnection (con);
 }
 
-void RKFrontendTransmitter::newProcessOutput () {
-	RK_TRACE (RBACKEND);
-
-	QString output = QString::fromLocal8Bit (backend->readAll ());
-	handleOutput (output, output.size (), ROutput::Warning);
-}
-
 void RKFrontendTransmitter::requestReceived (RBackendRequest* request) {
 	RK_TRACE (RBACKEND);
 
@@ -124,36 +111,18 @@ void RKFrontendTransmitter::requestReceived (RBackendRequest* request) {
 		ROutputList* list = request->output;
 		for (int i = 0; i < list->size (); ++i) {
 			ROutput *out = (*list)[i];
-			handleOutput (out->output, out->output.length (), out->type);
+
+			if (handleOutput (out->output, out->output.length (), out->type)) {
+				RKRBackendEvent* event = new RKRBackendEvent (new RBackendRequest (false, RBackendRequest::OutputStartedNotification));
+				qApp->postEvent (RKRBackendProtocolFrontend::instance (), event);
+			}
+
 			delete (out);
 		}
 		request->output = 0;
 		RK_ASSERT (request->synchronous);
 		writeRequest (request);	// to tell the backend, that we are keeping up. Also deletes the request.
 		return;
-	} else if (request->type == RBackendRequest::SyncOutput) {
-		RK_ASSERT (request->synchronous);
-
-		QString token = request->params["endtoken"].toString ();
-		writeRequest (request);
-
-		if (!token.isEmpty ()) {
-			QString buffer;
-
-			disconnect (backend, SIGNAL (readyReadStandardOutput ()), this, SLOT (newProcessOutput ()));
-			for (int i=5; i > 0; --i) {		// don't wait forever for the end-token.
-				buffer.append (QString::fromLocal8Bit (backend->readAll ()));
-				if (buffer.endsWith (token)) {
-					buffer = buffer.left (buffer.size () - token.size ());
-					break;
-				}
-				backend->waitForReadyRead (500);
-			}
-			connect (backend, SIGNAL (readyReadStandardOutput ()), this, SLOT (newProcessOutput ()));
-
-			if (!buffer.isEmpty ()) handleOutput (buffer, buffer.size (), ROutput::Warning);
-		}
-		return;
 	}
 
 	RKRBackendEvent* event = new RKRBackendEvent (request);
