@@ -96,6 +96,8 @@ extern "C" {
 #ifndef Q_WS_WIN
 #	include <signal.h>		// needed for pthread_kill
 #	include <pthread.h>		// seems to be needed at least on FreeBSD
+#	include <unistd.h>		// for non-blocking pipes
+#	include <fcntl.h>
 #endif
 
 void RK_scheduleIntr () {
@@ -381,6 +383,30 @@ int RReadConsoleWin (const char* prompt, char* buf, int buflen, int hist) {
 }
 #endif
 
+bool RKRBackend::fetchStdoutStderr (bool forcibly) {
+#ifndef Q_OS_WIN
+	if (!forcibly) {
+		if (!stdout_stderr_mutex.tryLock ()) return false;
+	} else {
+		stdout_stderr_mutex.lock ();
+	}
+
+	// it seems, setting this only once is not always enough.
+	fcntl (stdout_stderr_fd, F_SETFL, fcntl (stdout_stderr_fd, F_GETFL, 0) | O_NONBLOCK);
+	char buffer[1024];
+	while (true) {
+		int bytes = read (stdout_stderr_fd, buffer, 1023);
+		if (bytes <= 0) break;
+		buffer[bytes] = '\0';
+		// NOTE: we must not risk blocking inside handleOutput, while the stdout_stderr_mutex is locked!
+		handleOutput (current_locale_codec->toUnicode (buffer, bytes), bytes, ROutput::Warning, false);
+	}
+
+	stdout_stderr_mutex.unlock ();
+#endif
+	return true;
+}
+
 void RWriteConsoleEx (const char *buf, int buflen, int type) {
 	RK_TRACE (RBACKEND);
 
@@ -400,6 +426,8 @@ void RWriteConsoleEx (const char *buf, int buflen, int type) {
 		}
 	}
 
+	if (RKRBackend::this_pointer->killed == RKRBackend::AlreadyDead) return;	// this check is mostly for fork()ed clients
+	RKRBackend::this_pointer->fetchStdoutStderr (true);
 	RKRBackend::this_pointer->handleOutput (RKRBackend::this_pointer->current_locale_codec->toUnicode (buf, buflen), buflen, type == 0 ? ROutput::Output : ROutput::Warning);
 }
 
@@ -527,7 +555,7 @@ int RChooseFile (int isnew, char *buf, int len) {
 /* There are about one million possible entry points to editing / showing files. We try to cover them all, using the
 following bunch of functions (REditFilesHelper() and doShowEditFiles() are helpers, only) */
 
-void REditFilesHelper (QStringList files, QStringList titles, QString wtitle, RBackendRequest::RCallbackType edit, bool delete_files) {
+void REditFilesHelper (QStringList files, QStringList titles, QString wtitle, RBackendRequest::RCallbackType edit, bool delete_files, bool prompt) {
 	RK_TRACE (RBACKEND);
 
 	RK_ASSERT ((edit == RBackendRequest::ShowFiles) || (edit == RBackendRequest::EditFiles));
@@ -540,6 +568,7 @@ void REditFilesHelper (QStringList files, QStringList titles, QString wtitle, RB
 	request.params["files"] = QVariant (files);
 	request.params["titles"] = QVariant (titles);
 	request.params["wtitle"] = QVariant (wtitle);
+	request.params["prompt"] = QVariant (prompt);
 
 	RKRBackend::this_pointer->handleRequest (&request);
 }
@@ -547,30 +576,31 @@ void REditFilesHelper (QStringList files, QStringList titles, QString wtitle, RB
 int REditFiles (int nfile, const char **file, const char **title, const char *wtitle) {
 	RK_TRACE (RBACKEND);
 
-	REditFilesHelper (charPArrayToQStringList (file, nfile), charPArrayToQStringList (title, nfile), wtitle, RBackendRequest::EditFiles, false);
+	REditFilesHelper (charPArrayToQStringList (file, nfile), charPArrayToQStringList (title, nfile), wtitle, RBackendRequest::EditFiles, false, true);
 
 // default implementation seems to return 1 if nfile <= 0, else 1. No idea, what for. see unix/std-sys.c
 	return (nfile <= 0);
 }
 
-SEXP doShowEditFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP del, RBackendRequest::RCallbackType edit) {
+SEXP doShowEditFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP del, SEXP prompt, RBackendRequest::RCallbackType edit) {
 	RK_TRACE (RBACKEND);
 
 	QStringList file_strings = RKRSupport::SEXPToStringList (files);
 	QStringList title_strings = RKRSupport::SEXPToStringList (titles);
 	QString wtitle_string = RKRSupport::SEXPToString (wtitle);
 	bool del_files = RKRSupport::SEXPToInt (del, 0) != 0;
+	bool do_prompt = RKRSupport::SEXPToInt (prompt, 0) != 0;
 
 	RK_ASSERT (file_strings.size () == title_strings.size ());
 	RK_ASSERT (file_strings.size () >= 1);
 
-	REditFilesHelper (file_strings, title_strings, wtitle_string, edit, del_files);
+	REditFilesHelper (file_strings, title_strings, wtitle_string, edit, del_files, do_prompt);
 
 	return (R_NilValue);
 }
 
-SEXP doEditFiles (SEXP files, SEXP titles, SEXP wtitle) {
-	return (doShowEditFiles (files, titles, wtitle, R_NilValue, RBackendRequest::EditFiles));
+SEXP doEditFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP prompt) {
+	return (doShowEditFiles (files, titles, wtitle, R_NilValue, prompt, RBackendRequest::EditFiles));
 }
 
 int REditFile (const char *buf) {
@@ -583,14 +613,14 @@ int REditFile (const char *buf) {
 	return REditFiles (1, const_cast<const char**> (&buf), &title, editor);
 }
 
-SEXP doShowFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP delete_files) {
-	return (doShowEditFiles (files, titles, wtitle, delete_files, RBackendRequest::ShowFiles));
+SEXP doShowFiles (SEXP files, SEXP titles, SEXP wtitle, SEXP delete_files, SEXP prompt) {
+	return (doShowEditFiles (files, titles, wtitle, delete_files, prompt, RBackendRequest::ShowFiles));
 }
 
 int RShowFiles (int nfile, const char **file, const char **headers, const char *wtitle, Rboolean del, const char */* pager */) {
 	RK_TRACE (RBACKEND);
 
-	REditFilesHelper (charPArrayToQStringList (file, nfile), charPArrayToQStringList (headers, nfile), QString (wtitle), RBackendRequest::ShowFiles, (bool) del);
+	REditFilesHelper (charPArrayToQStringList (file, nfile), charPArrayToQStringList (headers, nfile), QString (wtitle), RBackendRequest::ShowFiles, (bool) del, true);
 
 // default implementation seems to returns 1 on success, 0 on failure. see unix/std-sys.c
 	return 1;
@@ -665,7 +695,8 @@ void RBusy (int busy) {
 
 // ############## R Standard callback overrides END ####################
 
-RKRBackend::RKRBackend () {
+// NOTE: stdout_stderr_mutex is recursive to support fork()s, better
+RKRBackend::RKRBackend () : stdout_stderr_mutex (QMutex::Recursive) {
 	RK_TRACE (RBACKEND);
 
 	current_locale_codec = QTextCodec::codecForLocale ();
@@ -864,25 +895,6 @@ SEXP doUpdateLocale () {
 	return R_NilValue;
 }
 
-SEXP doSyncOutput (SEXP flushstdout) {
-	RK_TRACE (RBACKEND);
-
-#if (!defined RKWARD_THREADED) && (!defined Q_OS_WIN)
-	const char* token = "##RKOutputEndTag3210723##";	// should be unique enough for practical purposes
-	bool doflushstdout = (RKRSupport::SEXPToInt (flushstdout) != 0);
-
-	RBackendRequest req (true, RBackendRequest::SyncOutput);
-	if (doflushstdout) req.params["endtoken"] = QString (token);
-	RKRBackend::this_pointer->handleRequest (&req);
-	if (doflushstdout) {
-		printf ("%s", token);
-		fflush (stdout);
-	}
-#endif
-
-	return R_NilValue;
-}
-
 // returns the MIME-name of the current locale encoding (from Qt)
 SEXP doLocaleName () {
 	RK_TRACE (RBACKEND);
@@ -950,6 +962,16 @@ bool RKRBackend::startR () {
 	FlushConsoleInputBuffer(GetStdHandle(STD_INPUT_HANDLE));
 #endif
 
+#ifndef Q_OS_WIN
+	// re-direct stdout / stderr to a pipe, so we can read output from system() calls
+	int pfd[2];
+	pipe (pfd);
+	dup2 (STDOUT_FILENO, STDERR_FILENO);		// single channel to avoid interleaving hell, for now.
+	dup2 (pfd[1], STDOUT_FILENO);
+	close (pfd[1]);
+	stdout_stderr_fd = pfd[0];
+#endif
+
 	setup_Rmainloop ();
 
 #ifndef Q_WS_WIN
@@ -990,12 +1012,11 @@ bool RKRBackend::startR () {
 		{ "rk.get.structure", (DL_FUNC) &doGetStructure, 4 },
 		{ "rk.get.structure.global", (DL_FUNC) &doGetGlobalEnvStructure, 3 },
 		{ "rk.copy.no.eval", (DL_FUNC) &doCopyNoEval, 3 },
-		{ "rk.edit.files", (DL_FUNC) &doEditFiles, 3 },
-		{ "rk.show.files", (DL_FUNC) &doShowFiles, 4 },
+		{ "rk.edit.files", (DL_FUNC) &doEditFiles, 4 },
+		{ "rk.show.files", (DL_FUNC) &doShowFiles, 5 },
 		{ "rk.dialog", (DL_FUNC) &doDialog, 6 },
 		{ "rk.update.locale", (DL_FUNC) &doUpdateLocale, 0 },
 		{ "rk.locale.name", (DL_FUNC) &doLocaleName, 0 },
-		{ "rk.sync.output", (DL_FUNC) &doSyncOutput, 1 },
 		{ 0, 0, 0 }
 	};
 	R_registerRoutines (R_getEmbeddingDllInfo(), NULL, callMethods, NULL, NULL);
@@ -1016,9 +1037,50 @@ bool RKRBackend::startR () {
 	return true;
 }
 
+#ifndef Q_OS_WIN
+static bool backend_was_forked = false;
+void prepareFork () {
+	RK_TRACE (RBACKEND);
+	if (!RKRBackendProtocolBackend::inRThread ()) return;
+
+	// we need to make sure that the transmitter thread does not hold a lock on the mutex!
+	RKRBackend::this_pointer->stdout_stderr_mutex.lock ();
+}
+
+void completeForkMaster () {
+	RK_TRACE (RBACKEND);
+	if (!RKRBackendProtocolBackend::inRThread ()) return;
+
+	RKRBackend::this_pointer->stdout_stderr_mutex.unlock ();
+
+	if (backend_was_forked) return;
+	backend_was_forked = true;
+
+	// Block SIGCLD in the main thread from now on. I don't fully understand, why, but otherwise, these signals
+	// interrupt the select() call in the fork()ing code of library(parallel)
+	sigset_t new_set;
+	sigemptyset (&new_set);
+	sigaddset (&new_set, SIGCLD);
+	pthread_sigmask (SIG_BLOCK, &new_set, NULL);
+
+//	This was used to show a warning message. Unfortunately, however, forks also occur on every popen (i.e. in system(..., intern=TRUE).
+//	RKRBackend::this_pointer->handlePlainGenericRequest (QStringList ("forkNotification"), false);
+	RK_DO (qDebug ("Backend process forked (for the first time, this session)"), RBACKEND, DL_WARNING);
+}
+
+void completeForkChild () {
+	RKRBackendProtocolBackend::instance ()->r_thread_id = QThread::currentThreadId();
+	RKRBackend::this_pointer->killed = RKRBackend::AlreadyDead;	// Not quite accurate, but disables all communication with the frontend
+}
+#endif
+
 void RKRBackend::enterEventLoop () {
 	RK_TRACE (RBACKEND);
 
+#ifndef Q_OS_WIN
+	pthread_atfork (prepareFork, completeForkMaster, completeForkChild);
+#endif
+
 	run_Rmainloop ();
 	// NOTE: Do NOT run Rf_endEmbeddedR(). It does more that we want. We rely on RCleanup, instead.
 }
@@ -1287,11 +1349,16 @@ RCommandProxy* RKRBackend::handleRequest (RBackendRequest *request, bool mayHand
 		return 0;
 	}
 
+	int i = 0;
 	while (!request->done) {
 		if (killed) return 0;
 		// NOTE: processX11Events() may, conceivably, lead to new requests, which may also wait for sub-commands!
 		processX11Events ();
-		if (!request->done) RKRBackendProtocolBackend::msleep (10);
+		// NOTE: sleeping and waking up again can be surprisingly CPU-intensive (yes: more than the event processing, above. I have profiled it).
+		// However, we really don't want to introduce too much delay, either.
+		// Thus, the logic is this: If there was no reply within 2 seconds, then probably we're waiting for a user event, and can afford some more
+		// latency (not too much, though, as we still need to process events).
+		if (!request->done) RKRBackendProtocolBackend::msleep (++i < 200 ? 10 : 50);
 	}
 
 	RCommandProxy* command = request->takeCommand ();
@@ -1353,6 +1420,10 @@ QStringList RKRBackend::handlePlainGenericRequest (const QStringList &parameters
 		request.params["call"] = dummy;
 	} else if (parameters.value (0) == "set.output.file") {
 		output_file = parameters.value (1);
+		if (parameters.length () > 2) {
+			RK_ASSERT (parameters.value (2) == "SILENT");
+			return QStringList ();		// For automated testing. The frontend should not be notified, here
+		}
 		request.params["call"] = parameters;
 	} else {
 		request.params["call"] = parameters;
