@@ -24,6 +24,8 @@
 
 #include "../debug.h"
 
+#define NAMED_CHILDREN_LIMIT 100000
+
 RKStructureGetter::RKStructureGetter (bool keep_evalled_promises) {
 	RK_TRACE (RBACKEND);
 
@@ -213,7 +215,6 @@ void RKStructureGetter::getStructureWorker (SEXP val, const QString &name, int a
 	}
 
 	if (RKRSupport::callSimpleBool (is_matrix_fun, value, R_BaseEnv)) type |= RObject::Matrix;
-	if (RKRSupport::callSimpleBool (is_array_fun, value, R_BaseEnv)) type |= RObject::Array;
 	if (RKRSupport::callSimpleBool (is_list_fun, value, R_BaseEnv)) type |= RObject::List;
 
 	if (type != 0) {
@@ -233,6 +234,8 @@ void RKStructureGetter::getStructureWorker (SEXP val, const QString &name, int a
 			else if (RKRSupport::callSimpleBool (is_numeric_fun, value, R_BaseEnv)) type |= RObject::Numeric;
 			else if (RKRSupport::callSimpleBool (is_character_fun, value, R_BaseEnv)) type |= RObject::Character;
 			else if (RKRSupport::callSimpleBool (is_logical_fun, value, R_BaseEnv)) type |= RObject::Logical;
+
+			if (RKRSupport::callSimpleBool (is_array_fun, value, R_BaseEnv)) type |= RObject::Array;
 		}
 	}
 	type |= add_type_flags;
@@ -336,6 +339,10 @@ void RKStructureGetter::getStructureWorker (SEXP val, const QString &name, int a
 		PROTECT (childnames_s);
 		QStringList childnames = RKRSupport::SEXPToStringList (childnames_s);
 		int childcount = childnames.size ();
+		if (childcount > NAMED_CHILDREN_LIMIT) {
+			RK_DO (qDebug ("object %s has %d named children. Will only retrieve the first %d", name.toLatin1().data (), childcount, NAMED_CHILDREN_LIMIT), RBACKEND, DL_WARNING);
+			childcount = NAMED_CHILDREN_LIMIT;
+		}
 
 		RData::RDataStorage children (childcount, 0);
 		for (int i = 0; i < childcount; ++i) {
