@@ -63,64 +63,95 @@
 	}
 }
 
-".rk.rkreply" <- NULL
-".rk.set.reply" <- function (x) .rk.rkreply <<- x
+".rk.set.reply" <- function (x) .rk.variables$.rk.rkreply <- x
 
 ".rk.do.call" <- function (x, args=NULL) {
 	.rk.set.reply (NULL)
 	.Call ("rk.do.command", c (x, args));
-	return (.rk.rkreply)
+	return (.rk.variables$.rk.rkreply)
 }
 
 ".rk.do.plain.call" <- function (x, args=NULL, synchronous=TRUE) {
 	.Call ("rk.do.generic.request", c (x, args), isTRUE (synchronous))
 }
 
+".rk.find.package.pluginmaps" <- function (package, all.maps=FALSE) {
+	if(isTRUE(all.maps)){
+		# look for all pluginmaps in the rkward folder
+		pluginmaps <- sapply(package, function(this.package){
+				dir(system.file("rkward", package=this.package), pattern="*.pluginmap", full.names=TRUE)
+			})
+	} else {
+		# check if a main .pluginmap file is provided
+		pluginmaps <- sapply(package, function(this.package){
+				system.file(file.path("rkward", paste(this.package, ".pluginmap", sep="")), package=this.package)
+			})
+	}
+	return(pluginmaps)
+}
+
+# Gather status information on installed and available packages.
+# Return value is used in class RKRPackageInstallationStatus of the frontend
+".rk.get.package.intallation.state" <- function () {
+	# fetch all status information
+	available <- .rk.cached.available.packages ()
+	inst <- installed.packages (fields="Title")
+	old <- as.data.frame (rk.old.packages (available=available), stringsAsFactors=FALSE)
+	new <- new.packages (instPkgs=inst, available=available)
+
+	# convert info to a more suitable format
+	available <- as.data.frame (available, stringsAsFactors=FALSE)
+	inst <- as.data.frame (inst, stringsAsFactors=FALSE)
+	oldinst <- match (paste (old$Package, old$LibPath), paste (inst$Package, inst$LibPath))	# convert package names to position with in the installed packages info
+	oldavail <- match (old$Package, available$Package)	# convert package names to position with in the available packages info
+	new <- match (new, available$Package)	# same for new packages
+
+	# as a side effect, we update the list of known installed packages in the frontend
+	.rk.do.plain.call ("updateInstalledPackagesList", sort (unique (as.character (inst$Package))), synchronous=FALSE)
+
+	list ("available" = list (available$Package, available$Title, available$Version, available$Repository, grepl ("rkward", available$Enhances)),
+		"installed" = list (inst$Package, inst$Title, inst$Version, inst$LibPath, grepl ("rkward", inst$Enhances)),
+		"new" = as.integer (new - 1),
+		"old" = list (as.integer (oldinst - 1), as.integer (oldavail - 1)),
+		"repos" = as.character (options("repos")$repos))
+}
+
 # package information formats may - according to the help - be subject to change. Hence this function to cope with "missing" values
 # also it concatenates everything to a single vector, so we can easily get the whole structure with a single call
 ".rk.get.installed.packages" <- function () {
-	x <- as.data.frame (installed.packages ())
-	try (titles <- as.data.frame (library ()$results)$Title)
-	if (length (titles) != dim (x)[1]) titles <- rep ("", dim (x)[1])
-	return (list (as.character (x$Package), as.character (titles), as.character (x$Version), as.character (x$LibPath)))
+	x <- as.data.frame(installed.packages(fields="Title"))
+	# does a package enhance RKWard, i.e. provide plugins?
+	enhance.rk <- ifelse(is.na(x$Enhances), FALSE, grepl("rkward", x$Enhances))
+
+	# as a side effect, we update the list of known installed packages in the frontend
+	.rk.do.plain.call ("updateInstalledPackagesList", sort (unique (as.character (x$Package))), synchronous=FALSE)
+	# check for pluginmaps only in packages which enhance RKWard
+	rk.load.pluginmaps (.rk.find.package.pluginmaps(x$Package[enhance.rk]), force.add=FALSE, force.reload=FALSE)
+
+	return(list(Package=as.character(x$Package), Title=as.character(x$Title), 
+		Version=as.character(x$Version), LibPath=as.character(x$LibPath),
+		EnhanceRK=as.logical(enhance.rk)))
 }
 
-# Here we concatenate everything (same as above) to get search results easily
-".rk.get.search.results" <- function (pattern, ...) {
-	H=as.data.frame (help.search(pattern, ...)$matches)
-	return(c(as.vector(H$topic),as.vector(H$title),as.vector(H$Package)))
-}
-
-".rk.available.packages.cache" <- NULL
 # This function works like available.packages (with no arguments), but does simple caching of the result, and of course uses a cache if available. Cache is only used, if it is less than 1 hour old, and options("repos") is unchanged.
 ".rk.cached.available.packages" <- function () {
 	x <- NULL
-	if (exists (".rk.available.packages.cache") && (!is.null (.rk.available.packages.cache))) {
-		if (.rk.available.packages.cache$timestamp > (Sys.time () - 3600)) {
-			if (all (.rk.available.packages.cache$repos$repos == options ("repos")$repos)) {
-				x <- .rk.available.packages.cache$cache
+	if (exists ("available.packages.cache", envir=.rk.variables) && (!is.null (.rk.variables$available.packages.cache))) {
+		if (.rk.variables$available.packages.cache$timestamp > (Sys.time () - 3600)) {
+			if (all (.rk.variables$available.packages.cache$repos$repos == options ("repos")$repos)) {
+				x <- .rk.variables$available.packages.cache$cache
 			}
 		}
 	}
 
 	if (is.null(x)) {
-		x <- available.packages()
-		.rk.available.packages.cache <<- list (cache = x, timestamp = Sys.time (), repos = options ("repos"))
+		x <- available.packages(fields="Title")
+		.rk.variables$available.packages.cache <- list (cache = x, timestamp = Sys.time (), repos = options ("repos"))
 	}
 
 	return (x)
 }
 
-".rk.get.old.packages" <- function () {
-	x <- old.packages (available=.rk.cached.available.packages ())
-	return (list (as.character (x[,"Package"]), as.character (x[,"LibPath"]), as.character (x[,"Installed"]), as.character (x[,"ReposVer"]), rk.make.repos.string ()))
-}
-
-".rk.get.available.packages" <- function () {
-	x <- .rk.cached.available.packages ()
-	return (list (as.character (x[,1]), as.character (x[,2]), rk.make.repos.string ()))
-}
-
 "require" <- function (package, quietly = FALSE, character.only = FALSE, ...)
 {
 	if (!character.only) {
@@ -201,7 +232,7 @@
 ".rk.unwatch.symbol" <- function (k) {
 	rm (list=k, envir=globalenv ())
 
-	k <<- .rk.watched.symbols$k
+	assign (k, .rk.watched.symbols$k, envir=globalenv ())
 
 	rm (k, envir=.rk.watched.symbols);
 
@@ -296,8 +327,6 @@
 	ret
 }
 
-".rk.output.html.file" <- NULL
-
 "Sys.setlocale" <- function (category = "LC_ALL", locale = "", ...) {
 	if (category == "LC_ALL" || category == "LC_CTYPE" || category == "LANG") {
 		allow <- .rk.do.plain.call ("preLocaleChange", NULL)
@@ -334,14 +363,14 @@ formals (setwd) <- formals (base::setwd)
 
 # Start recording commands that are submitted from rkward to R.
 # filename: filename to write to (file will be truncated!).
-# include.sync.commands: Should internal synchronisation commands be included?
+# include.all: By default, some types of command are filtered (internal synchronisation commands, and run again links). Should these be included?
 # To stop recording, supply NULL or "" as filename
 # Currently used for the purpose of automated testing, only. Perhaps in the future
 # this or a similar mechanism could also be added as a user feature.
-"rk.record.commands" <- function (filename, include.sync.commands = FALSE) {
+"rk.record.commands" <- function (filename, include.all = FALSE) {
 	if (is.null (filename)) filename = ""
 
-	res <- .rk.do.plain.call ("recordCommands", c(as.character (filename), if (include.sync.commands) "include.sync" else "normal"))
+	res <- .rk.do.plain.call ("recordCommands", c(as.character (filename), if (include.all) "include.all" else "normal"))
 
 	if (!length (res)) invisible (TRUE)
 	else {
@@ -350,73 +379,14 @@ formals (setwd) <- formals (base::setwd)
 	}
 }
 
-# retrieve the (expected) "base" url of help files. Most importantly this will be a local port for R 2.10.0 and above, but a local directory for 2.9.x and below. As a side effect, in R 2.10.0 and above, the dynamic help server is started.
-".rk.getHelpBaseUrl" <- function () {
-	port <- NA
-	if (compareVersion (as.character (getRversion()), "2.10.0") >= 0) {
-		try ({
-			port <- tools::startDynamicHelp ()
-		})
-		if (is.na (port)) {
-			try ({
-				port <- tools:::httpdPort
-			})
-		}
-	}
-	if (is.na (port)) {
-		return (paste ("file://", R.home (), sep=""))
-	}
-	return (paste ("http://127.0.0.1", port, sep=":"))
-}
+# General purpose storage environment (which will hopefully never get locked by R)
+".rk.variables" <- new.env ()
+.rk.variables$.rk.active.device <- 1
+.rk.variables$.rk.output.html.file <- NULL
+.rk.variables$.rk.rkreply <- NULL
+.rk.variables$available.packages.cache <- NULL
 
-# a simple wrapper around help() that makes it easier to detect in code, whether help was found or not.
-# used from RKHelpSearchWindow::getFunctionHelp
-".rk.getHelp" <- function (...) {
-	if (compareVersion (as.character (getRversion()), "2.10.0") >= 0) {
-		res <- help (..., help_type="html")
-	} else {
-		res <- help (..., chmhelp=FALSE, htmlhelp=TRUE)
-	}
-	if (!length (as.character (res))) {	# this seems undocumented, but it is what utils:::print.help_files_with_topic checks
-		show (res)
-		stop ("No help found")
-	}
-	show (res)
-	invisible (TRUE)
-}
-
-# Tries to replace a function inside its environemnt/namespace.
-# Function formals are copied from the original.
-# A backup of the original is stored as rkward::.rk.FUNCTIONNAME.default
-"rk.replace.function" <- function (functionname, environment, replacement, copy.formals=TRUE) {
-	original <- get (functionname, envir=environment, inherits=FALSE)
-
-	# create a backup
-	backupname <- paste (".rk.", functionname, ".default", sep="")
-	assign (backupname, original, envir=as.environment ("package:rkward"))
-
-	if (copy.formals) formals (replacement) <- formals (original)
-	environment (replacement) <- environment (original)
-	assign (functionname, replacement, envir=as.environment ("package:rkward"))
-	try (
-		if (bindingIsLocked (functionname, environment)) {
-			unlockBinding (functionname, environment)
-			on.exit (lockBinding (functionname, environment))
-		}
-	)
-	try (
-		if (isNamespace (environment)) {
-			assignInNamespace (functionname, replacement, ns=environment)
-		} else {
-			assignInNamespace (functionname, replacement, envir=environment)
-		}
-	)
-	try (
-		assign (functionname, replacement, envir=environment)
-	)
-
-	invisible (NULL)
-}
+".rk.backups" <- new.env ()
 
 # where masking is not enough, we need to assign in the original environment / namespace. This can only be done after package loading,
 # so we have a separate function for that.
@@ -452,7 +422,7 @@ formals (setwd) <- formals (base::setwd)
 			}
 
 			# for text list, use the default implementation
-			eval (body (rkward::.rk.select.list.default))
+			eval (body (.rk.backups$select.list))
 		})
 
 	rk.replace.function ("menu", as.environment ("package:utils"),
@@ -463,36 +433,9 @@ formals (setwd) <- formals (base::setwd)
 			}
 
 			# for text menus, use the default implementation
-			eval (body (.rk.menu.default))
+			eval (body (.rk.backups$menu))
 		})
 
-	## Add output synchronisation across system(), and system2() calls.
-	rk.replace.function ("system", base::.BaseNamespaceEnv,
-		function () {
-			if (!exists ("ignore.stdout", inherits=FALSE)) ignore.stdout <- FALSE	# ignore.stdout was introduced in R 2.12.0
-
-			if (!(intern || (ignore.stdout && ignore.stderr))) {
-				.Call ("rk.sync.output", 0)
-				on.exit (.Call ("rk.sync.output", 1), TRUE)
-			}
-
-			eval (body (.rk.system.default))
-		})
-
-	# NOTE: system2 was not introduced before R 2.12.0 (or was it 2.11.0?)
-	if (exists ("system2", base::.BaseNamespaceEnv)) {
-		rk.replace.function ("system2", base::.BaseNamespaceEnv,
-			function () {
-				if (stdout != "" || stderr != "") {
-					.Call ("rk.sync.output", 0)
-					on.exit (.Call ("rk.sync.output", 1), TRUE)
-				}
-				eval (body (.rk.system2.default))
-			})
-	}
-
 	# call separate assignments functions:
 	if (exists (".rk.fix.assignments.graphics")) eval (body (.rk.fix.assignments.graphics)) # internal_graphics.R
 }
-
-.rk.active.device <- 1
