@@ -8,7 +8,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 	if (!is.numeric (height)) height <- 480
 	if (is.null (device.type)) device.type <- "PNG"	# default behavior is PNG for now
 
-	assign (".rk.active.device", dev.cur (), pos = "package:rkward")
+	assign (".rk.active.device", dev.cur (), .rk.variables)
 
 	ret <- NULL
 	if (device.type == "PNG") {
@@ -50,11 +50,42 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 	
 	# dev.off () sets dev.next () as active, which may not have been active before rk.graph.on was called;
 	# so reset the correct device as active:
-	i <- get (".rk.active.device", pos = "package:rkward")
+	i <- get (".rk.active.device", .rk.variables)
 	if ((!is.null (i)) && (i %in% dev.list ())) ret <- dev.set (i)
 	ret
 }
 
+#' Device for printing using the KDE print dialog
+#' 
+#' Creates a device operating on a temporary file (internally a
+#' \code{\link{postscript}}() device). When the device is closed, it is
+#' printed, automatically, using the KDE print dialog (if installed).
+#' 
+#' Typically this device is used with \code{\link{dev.print}}, as shown in the
+#' example, below.
+#' 
+#' @param ... arguments are passed to \code{\link{postscript}}
+#' @return Returns the name of the underlying temporary file, invisibly.
+#' @author Thomas Friedrichsmeier \email{rkward-devel@@lists.sourceforge.net}
+#' @seealso \code{\link{postscript}}, \code{\link{dev.print}},
+#'   \code{\link{rk.graph.on}}
+#' @keywords utilities device
+#' @rdname rk.printer.device
+#' @examples
+#' 
+#' ## Not run:
+#' plot (rnorm (10))
+#' dev.print (rk.printer.device)
+#' 
+# Produces a temporary postscript file and opens a print dialog for it
+# Parameters are passed to postscript(), but typically this is simply used as
+#   dev.print(rk.print.preview)
+"rk.printer.device" <- function(...) {
+	tf <- paste (tempfile (), "ps", sep=".")	# NOTE: Up to R 2.12.x, tempfile() did not have "fileext" parameter
+	postscript (file = tf, ...)
+	.rk.variables$.rk.printer.devices[[as.character (dev.cur ())]] <- tf
+}
+
 "rk.duplicate.device" <- function (devId = dev.cur ())
 {
 	dev.set (devId)
@@ -163,7 +194,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 		
 		# all open screen devices
 		.osd <- which (names (dev.list ()) %in% deviceIsInteractive ()) + 1
-		.opd <- unlist (.rk.preview.devices)
+		.opd <- unlist (.rk.variables$.rk.preview.devices)
 		# to be managed devices:
 		if (length (.opd) > 0) .osd <-.osd [!(.osd %in% .opd)]
 		if (length (.osd) == 0) return (invisible ())
@@ -171,14 +202,14 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 		d.cur <- dev.cur ()
 		histPositions <<- list ("1" = .hP.template)
 		for (d in as.character (.osd)) {
-			.rk.dev.set.default (as.numeric (d))
+			.rk.backups$dev.set (as.numeric (d))
 			if (is.null (recordPlot ()[[1]])) # empty device
 				histPositions [[d]] <<- .hP.template
 			else
 				histPositions [[d]] <<- modifyList(.hP.template, 
 					list (is.this.plot.new = TRUE, is.this.dev.new = FALSE, pkg = "unknown"))
 		}
-		.rk.dev.set.default (d.cur)
+		.rk.backups$dev.set (d.cur)
 		.set.hP.names ()
 	}
 	onDelDevice <- function (devId = dev.cur())
@@ -315,9 +346,9 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 		unsplot <- NULL
 		unsplot.ls <- NULL
 		if (pkg %in% c("graphics", "unknown")) {
-			.rk.dev.set.default (as.numeric (devId))
+			.rk.backups$dev.set (as.numeric (devId))
 			try (unsplot <- recordPlot(), silent=TRUE)
-			.rk.dev.set.default (devId.cur)
+			.rk.backups$dev.set (devId.cur)
 		} else if  (pkg == "lattice") {
 			unsplot <- histPositions [[devId]]$plot
 			unsplot.ls <- histPositions [[devId]]$tlo.ls
@@ -502,7 +533,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 		
 		devId <- as.character (devId)
 		cur.devId <- dev.cur ()
-		.rk.dev.set.default (as.numeric(devId))
+		.rk.backups$dev.set (as.numeric(devId))
 		
 		st <- .sP.index [[n]]
 		pkg <- savedPlots [[st]]$pkg
@@ -523,7 +554,7 @@ rk.graph.on <- function (device.type=getOption ("rk.graphics.type"), width=getOp
 		histPositions [[devId]] <<- modifyList (.hP.template, 
 			list (is.this.plot.new = FALSE, is.this.dev.new = FALSE, pos.prev = n, pos.cur = n, pkg = pkg, 
 				call = savedPlots [[st]]$call, plot = savedPlots [[st]]$plot, tlo.ls = savedPlots [[st]]$tlo.ls))
-		.rk.dev.set.default (cur.devId)
+		.rk.backups$dev.set (cur.devId)
 		invisible()
 	}
 	
