@@ -7,6 +7,7 @@
 #' @slot expect_error Expect errors
 #' @slot libraries A charcter vector naming needed libraries
 #' @slot files A character vector naming needed files, path relative to the test standards directory
+#' @slot record.all.commands Should synchronization commands and commands to generate run-again-links be included in the command recording? Generally, this should be FALSE (the default).
 #' @name RKTest
 #' @import methods
 #' @keywords classes
@@ -15,8 +16,8 @@
 #' @rdname RKTest-class
 
 setClass ("RKTest",
-		representation (id="character", call="function", fuzzy_output="logical", expect_error="logical", libraries="character", files="character"),
-		prototype(character(0), id=NULL, call=function () { stop () }, fuzzy_output=FALSE, expect_error=FALSE, libraries=character(0), files=character(0)),
+		representation (id="character", call="function", fuzzy_output="logical", expect_error="logical", libraries="character", files="character", record.all.commands="logical"),
+		prototype(character(0), id=NULL, call=function () { stop () }, fuzzy_output=FALSE, expect_error=FALSE, libraries=character(0), files=character(0), record.all.commands=FALSE),
 		validity=function (object) {
 			if (is.null (object@id)) return (FALSE)
 			return (TRUE)
