@@ -69,8 +69,12 @@ RKConsole::RKConsole (QWidget *parent, bool tool_window, const char *name) : RKM
 	layout->setContentsMargins (0, 0, 0, 0);
 
 	// create a Kate-part as command-editor
-	KTextEditor::Editor* editor = KTextEditor::editor("katepart");
-	RK_ASSERT (editor);
+	KTextEditor::Editor* editor = KTextEditor::editor ("katepart");
+	if (!editor) {
+		RK_ASSERT (false);
+		KMessageBox::error (this, i18n ("The 'katepart' component could not be loaded. RKWard cannot run without katepart, and will exit, now. Please install katepart, and try again."), i18n ("'katepart' component could not be found"));
+		exit (1);
+	}
 
 	doc = editor->createDocument (this);
 	view = doc->createView (this);
