@@ -2,7 +2,7 @@
 			rkward.h  -  description
 			-------------------
 begin                : Tue Oct 29 20:06:08 CET 2002 
-copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010 by Thomas Friedrichsmeier 
+copyright            : (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011 by Thomas Friedrichsmeier 
 email                : tfry@users.sourceforge.net
 ***************************************************************************/
 
@@ -25,25 +25,14 @@ email                : tfry@users.sourceforge.net
 
 class QLabel;
 class QCloseEvent;
-// forward declaration of the RKward classes
-class RSettings;
-class RKMenu;
-class RObjectBrowser;
-class RKSettingsModule;
-class RKSettings;
-class RInterface;
-class RKEditorManager;
-class RKMenuList;
-class RKCommandEditorWindow;
-class KMdiToolViewAccessor;
-class RKMDIWindow;
+class KActionMenu;
 class RKTopLevelWindowGUI;
 class KRecentFilesAction;
 class KSqueezedTextLabel;
+class QAction;
 
 struct RKWardStartupOptions {
 	KUrl initial_url;	/**< The workspace file to load on startup. If empty, show a dialog asking what to do. **/
-	bool no_stack_check;	/**< Disable R C stack checking */
 	QString evaluate;	/**< R code to run after startup */
 };
 
@@ -73,15 +62,9 @@ public:
 
 	static RKWardMainWindow *getMain () { return rkward_mainwin; };
 
-/** return pointer to startup options. WARNING: The options are deleted shortly after startup */
-	static RKWardStartupOptions* getStartupOptions () { return getMain()->startup_options; };
-	static void discardStartupOptions () {
-		delete (getMain()->startup_options);
-		getMain()->startup_options = 0;
-	};
-
 /** (try to) close all windows, and ask whether it is ok to quit */
 	bool doQueryQuit ();
+	void lockGUIRebuild (bool lock);
 protected:
 	void openWorkspace (const KUrl &url);
 	/** save Options/Settings. Includes general Options like all bar positions and status as well as the geometry and the recent file list */
@@ -93,6 +76,8 @@ protected:
 	/** sets up the statusbar for the main window by initialzing a statuslabel.
 	*/
 	void initStatusBar();
+	/** sets up the various tool windows, and starts the R engine */
+	void initToolViewsAndR ();
 	/** reimplemented from KMainWindow to call our doQueryClose (), and then (if quitting was not cancelled), invoke an RKQuitAgent to wait for the R-backend to finish up before actually quitting. */
 	virtual void closeEvent (QCloseEvent *e);
 signals:
@@ -117,6 +102,8 @@ public slots:
 	/** Reimplemented from KParts::MainWindow to be more pretty
 	* @param text the text that is displayed in the statusbar
 	*/
+	void slotCancelAllCommands ();
+	void configureCarbonCopy ();
 	void slotSetStatusBarText (const QString &text);
 /** Basically a shortcut to slotSetStatusBarText (QString::null). Needed as a slot without parameters. */
 	void slotSetStatusReady () { slotSetStatusBarText (QString::null); };
@@ -167,12 +154,10 @@ private:
 	KAction* fileSaveWorkspace;
 	KAction* fileSaveWorkspaceAs;
 	KAction* fileQuit;
+	KAction* interrupt_all_commands;
 	KAction* close_all_editors;
 	KAction* new_data_frame;
 	KAction* new_command_editor;
-	
-	KAction* editUndo;
-	KAction* editRedo;
 
 	KAction* window_close_all;
 	KAction* window_detach;
@@ -183,8 +168,11 @@ private:
 	KAction* edit_menu_dummy;
 	/** used so that if the menu is empty, there is a note in it, explaining that fact */
 	KAction* view_menu_dummy;
-	/** used so that if the menu is empty, there is a note in it, explaining that fact */
-	KAction* run_menu_dummy;
+
+	KAction* proxy_export, *proxy_import;
+	KActionMenu* save_any_action;
+	QAction* save_actions_plug_point;
+	QList<QPointer <QAction> > plugged_save_actions;
 
 	friend class RKSettingsModule;
 	friend class RKSettingsModulePlugins;
@@ -209,6 +197,7 @@ private:
 	void updateCWD ();
 
 	RKTopLevelWindowGUI *toplevel_actions;
+	bool gui_rebuild_locked;
 };
 
 #endif // RKWARD_H
