@@ -1,26 +1,27 @@
 _script_output = "";
-function echo (text) {
+/* NOTE: for compatibility with Kross, we can't write this as 'function echo (text)...', somehow. */
+echo = function (text) {
 	_script_output += text;
 }
 
-function printIndented (indentation, lines) {
+printIndented = function (indentation, lines) {
 	/** More complex than may seem necessary at first glance. The point is that the
-	final linebreak in the input should *not* be replaced by linebreak + indentation. */
+	final line break in the input should *not* be replaced by line break + indentation. */
 	echo (indentation + lines.replace (/\n(.)/g, "\n" + indentation + "$1"));
 }
 
-function noquote (text) {
+noquote = function (text) {
 	var ret = new String (text);
 	ret.noquote = 1;
 	return (ret);
 }
 
-function quote (text) {
+quote = function (text) {
 	if (text.noquote) return text;
 	return ("\"" + text.replace (/\"/g, "\\\"") + "\"");
 }
 
-function makeHeaderCode (title, parameters) {
+makeHeaderCode = function (title, parameters) {
 	echo ("rk.header(" + quote (title));
 	if (parameters.length) {
 		echo (", parameters=list(");
@@ -36,55 +37,55 @@ function makeHeaderCode (title, parameters) {
 	echo (")\n");
 }
 
-function getValue (id) {
+getValue = function (id) {
 	return (_RK_backend.getValue (id));
 }
 
-function printValue (id) {
+printValue = function (id) {
 	echo (getValue (id));
 }
 
-function include (file) {
+include = function (file) {
 	_RK_backend.includeFile (file);
 }
 
-function flushOutput () {
+flushOutput = function () {
 	var string = _script_output;
 	_script_output = "";
 	return (string);
 }
 
-function do_preprocess () {
+do_preprocess = function () {
 	if (typeof (preprocess) == "undefined") return ("");
 	preprocess ();
 	return (flushOutput ());
 }
 
-function do_calculate () {
+do_calculate = function () {
 	if (typeof (calculate) == "undefined") return ("");
 	calculate ();
 	return (flushOutput ());
 }
 
-function do_printout () {
+do_printout = function () {
 	if (typeof (printout) == "undefined") return ("");
 	printout ();
 	return (flushOutput ());
 }
 
-function do_preview () {
+do_preview = function () {
 	if (typeof (preview) == "undefined") return ("");
 	preview ();
 	return (flushOutput ());
 }
 
 // for compatibility with the converted PHP code
-function trim (text) {
+trim = function (text) {
 	var ret = text.replace (/^\s*/, "").replace (/\s*$/, "");
 	return (ret);
 }
 
-function split (by, text) {
+split = function (by, text) {
 	return (text.split (by));
 }
 
