@@ -20,6 +20,7 @@
 #include <qobject.h>
 
 #include <qstring.h>
+#include <QLinkedList>
 
 class RKComponentPropertyCode;
 
@@ -79,7 +80,7 @@ protected:
 	/// whether command has finished
 		bool complete;
 	};
-	QValueList<ScriptCommand *> command_stack;
+	QLinkedList<ScriptCommand *> command_stack;
 
 	int current_flags;
 	int current_type;
