@@ -97,7 +97,7 @@ void SimpleBackend::tryNextFunction () {
 		else if (current_type == Printout) current_template = printout_template;
 		else if (current_type == Calculate) current_template = calculate_template;
 		else if (current_type == Preview) current_template = preview_template;
-		template_sep = current_template.find ("!!!");
+		template_sep = current_template.indexOf ("!!!");
 
 		if (template_sep < 0) {
 			commandFinished ("");
@@ -111,12 +111,12 @@ void SimpleBackend::tryNextFunction () {
 void SimpleBackend::processCall () {
 	RK_TRACE (PHP);
 
-	int next_token = current_template.find ("$$$", template_pos);
+	int next_token = current_template.indexOf ("$$$", template_pos);
 	if (next_token < 0) next_token = template_sep;
 	if (next_token > template_sep) next_token = template_sep;
 
 	if (next_token < template_sep) {
-		int token_end = current_template.find ("$$$", next_token + 3);
+		int token_end = current_template.indexOf ("$$$", next_token + 3);
 		RK_ASSERT (token_end >= 0);
 		QString token = current_template.mid (next_token + 3, token_end - (next_token + 3));
 		template_pos = token_end + 3;
@@ -143,15 +143,15 @@ void SimpleBackend::finishCall (const QString &conditions) {
 	int pos = 3;
 	int max = conds.length ();
 	do {
-		int cond_end = conds.find ("!?!", pos);
+		int cond_end = conds.indexOf ("!?!", pos);
 		if (cond_end < 0) cond_end = max;
 		QString condition = conds.mid (pos, cond_end - pos);
 
-		int if_end = condition.find ("!:!");
+		int if_end = condition.indexOf ("!:!");
 		RK_ASSERT (if_end >= 0);
 		QString if_part = condition.left (if_end);
 
-		int if_mid = if_part.find ("!=!");
+		int if_mid = if_part.indexOf ("!=!");
 		RK_ASSERT (if_mid >= 0);
 		QString if_compare = if_part.left (if_mid);
 
