@@ -2,7 +2,7 @@
                           rksettingsmoduleconsole  -  description
                              -------------------
     begin                : Sun Oct 16 2005
-    copyright            : (C) 2005, 2006, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2005, 2006, 2007, 2009 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -22,8 +22,8 @@
 #include <qnamespace.h>
 
 class QCheckBox;
+class QComboBox;
 class KIntSpinBox;
-class RKConsole;
 
 /**
 Settings module for the console. Allows you to configure whether to store command history, command history length. Future extensions: color for warnings/errors, etc.
@@ -48,7 +48,12 @@ public:
 	static uint maxHistoryLength () { return max_history_length; };
 	static uint maxConsoleLines () { return max_console_lines; };
 	static bool pipeUserCommandsThroughConsole () { return pipe_user_commands_through_console; };
-	static bool addPipedCommandsToHistory () { return add_piped_commands_to_history; };
+	enum PipedCommandsHistoryMode {
+		DontAdd = 0,
+		AddSingleLine = 1,
+		AlwaysAdd = 2
+	};
+	static PipedCommandsHistoryMode addPipedCommandsToHistory () { return add_piped_commands_to_history; };
 	/** Given the button state, return whether the command history should be navigated context sensitive or insensitive
 	@param current_state the current button state
 	@returns true, if a the search should be context sensitive, false for a normal search */
@@ -67,13 +72,13 @@ private:
 	static uint max_history_length;
 	static uint max_console_lines;
 	static bool pipe_user_commands_through_console;
-	static bool add_piped_commands_to_history;
+	static PipedCommandsHistoryMode add_piped_commands_to_history;
 	static bool context_sensitive_history_by_default;
 
 	QCheckBox *save_history_box;
 	QCheckBox *reverse_context_mode_box;
 	QCheckBox *pipe_user_commands_through_console_box;
-	QCheckBox *add_piped_commands_to_history_box;
+	QComboBox *add_piped_commands_to_history_box;
 	KIntSpinBox *max_history_length_spinner;
 	KIntSpinBox *max_console_lines_spinner;
 };
