@@ -22,11 +22,6 @@
 #include <kmessagebox.h>
 #include <khbox.h>
 #include <kdeversion.h>
-#ifdef RKWARD_USE_KNS3
-#	include <knewstuff3/downloaddialog.h>
-#else
-#	include <knewstuff2/engine.h>
-#endif
 #include <ktar.h>
 #include <kzip.h>
 #include <kio/deletejob.h>
@@ -45,11 +40,13 @@
 #include "../misc/multistringselector.h"
 #include "../misc/rkcommonfunctions.h"
 #include "../misc/rkspinbox.h"
+#include "rksettingsmodulegeneral.h"
 
 #include "../debug.h"
 
 // static members
 QStringList RKSettingsModulePlugins::plugin_maps;
+QStringList RKSettingsModulePlugins::known_plugin_maps;
 RKSettingsModulePlugins::PluginPrefs RKSettingsModulePlugins::interface_pref;
 bool RKSettingsModulePlugins::show_code;
 int RKSettingsModulePlugins::code_size;
@@ -116,11 +113,6 @@ RKSettingsModulePlugins::RKSettingsModulePlugins (RKSettings *gui, QWidget *pare
 	connect (map_choser, SIGNAL (listChanged ()), this, SLOT (settingChanged ()));
 	main_vbox->addWidget (map_choser);
 
-	main_vbox->addSpacing (2*RKGlobals::spacingHint ());
-	button = new QPushButton (i18n ("Install or uninstall add-on plugin packs"), this);
-	main_vbox->addWidget (button);
-	connect (button, SIGNAL (clicked()), this, SLOT (downloadPlugins()));
-
 	main_vbox->addStretch ();
 }
 
@@ -157,6 +149,7 @@ void RKSettingsModulePlugins::applyChanges () {
 	show_code = show_code_box->isChecked ();
 	code_size = code_size_box->intValue ();
 
+	fixPluginMapLists ();
 	RKWardMainWindow::getMain ()->initPlugins();
 }
 
@@ -170,6 +163,7 @@ void RKSettingsModulePlugins::saveSettings (KConfig *config) {
 
 	KConfigGroup cg = config->group ("Plugin Settings");
 	cg.writeEntry ("Plugin Maps", plugin_maps);
+	cg.writeEntry ("All known plugin maps", known_plugin_maps);
 	cg.writeEntry ("Interface Preferences", static_cast<int> (interface_pref));
 	cg.writeEntry ("Code display default", show_code);
 	cg.writeEntry ("Code display size", code_size);
@@ -180,58 +174,62 @@ void RKSettingsModulePlugins::loadSettings (KConfig *config) {
 
 	KConfigGroup cg = config->group ("Plugin Settings");
 	plugin_maps = cg.readEntry ("Plugin Maps", QStringList ());
+	known_plugin_maps = cg.readEntry ("All known plugin maps", QStringList ());
 	if (plugin_maps.isEmpty ()) {
 		plugin_maps.append (RKCommonFunctions::getRKWardDataDir () + "/all.pluginmap");
 	}
+	fixPluginMapLists ();
 
 	interface_pref = static_cast<PluginPrefs> (cg.readEntry ("Interface Preferences", static_cast<int> (PreferRecommended)));
 	show_code = cg.readEntry ("Code display default", false);
-	code_size = cg.readEntry ("Code display size", 40);
+	code_size = cg.readEntry ("Code display size", 250);
+	if (RKSettingsModuleGeneral::storedConfigVersion () <= RKSettingsModuleGeneral::RKWardConfig_Pre0_5_7) {
+		if (code_size == 40) code_size = 250;	// previous default untouched.
+	}
 }
 
-void RKSettingsModulePlugins::downloadPlugins () {
+// static
+void RKSettingsModulePlugins::registerPluginMaps (const QStringList &maps, bool force_add, bool force_reload) {
 	RK_TRACE (SETTINGS);
 
-	QStringList oldmaps = plugin_maps;
-
-#ifdef RKWARD_USE_KNS3
-	KNS3::DownloadDialog dialog ("rkward.knsrc", 0);
-	dialog.exec ();
-	KNS3::Entry::List list = dialog.changedEntries ();
-#else
-	KNS::Engine engine (0);
-	if (!engine.init ("rkward.knsrc")) return;
-	KNS::Entry::List list = engine.downloadDialogModal (this);
-#endif
-
-	for (int i = 0; i < list.size (); ++i) {
-#ifdef RKWARD_USE_KNS3
-		QStringList installed_files = list[i].installedFiles ();
-		QStringList uninstalled_files = list[i].uninstalledFiles ();
-#else
-		QStringList installed_files = list[i]->installedFiles ();
-		QStringList uninstalled_files = list[i]->uninstalledFiles ();
-#endif
-		foreach (const QString inst, installed_files) {
-			installPluginPack (inst);
-		}
-		foreach (const QString inst, uninstalled_files) {
-			uninstallPluginPack (inst);
+	QStringList added;
+	foreach (const QString &map, maps) {
+		if (map.isEmpty ()) continue;
+		if (known_plugin_maps.contains (map)) {
+			if (!force_add) continue;
+		} else {
+			known_plugin_maps.append (map);
 		}
+
+		if (plugin_maps.contains (map)) continue;
+		plugin_maps.append (map);
+		added.append (map);
 	}
 
-	// new pluginmaps were already added in installPluginPack. Now let's check, whether there any to be removed:
+	if (force_reload || (!added.isEmpty ())) {
+		// NOTE: Do this *before* showing the dialog, which is modal, i.e. has an event loop. Otherwise, subsequent calls e.g. to rk.call.plugin() could sneak in front of this
+		RKWardMainWindow::getMain ()->initPlugins();
+	}
+
+	if (!added.isEmpty ()) {
+		KMessageBox::informationList (RKWardMainWindow::getMain (), i18n ("New RKWard plugin packs (listed below) have been found, and have been activated, automatically. To de-activate selected plugin packs, use Settings->Configure RKWard->Plugins."), added, i18n ("New plugins found"), "new_plugins_found");
+	}
+}
+
+void RKSettingsModulePlugins::fixPluginMapLists () {
+	RK_TRACE (SETTINGS);
+
 	for (int i = 0; i < plugin_maps.size (); ++i) {
 		QFileInfo info (plugin_maps[i]);
 		if (!info.isReadable ()) {
+			known_plugin_maps.removeAll (plugin_maps[i]);
 			plugin_maps.removeAt (i);
 			--i;
 		}
 	}
 
-	if (plugin_maps != oldmaps) {
-		map_choser->setValues (plugin_maps);
-		change ();
+	foreach (const QString &map, plugin_maps) {
+		if (!known_plugin_maps.contains (map)) known_plugin_maps.append (map);
 	}
 }
 
