@@ -51,26 +51,35 @@ public:
 	static PluginPrefs getInterfacePreference () { return interface_pref; };
 	static bool showCodeByDefault () { return show_code; };
 	static int defaultCodeHeight () { return code_size; };
+	/** register a list of available plugin-maps (which may or may not already be known). New maps are activated, automatically.
+	 * @param maps Plugin maps (filenames) to add
+	 * @param force_add If true, maps are added, even if they are not "new", and had previously been disabled by the user
+	 * @param force_reload If true, plugin maps are always reloaded, even if no maps were added
+	 */
+	static void registerPluginMaps (const QStringList &maps, bool force_add, bool force_reload);
 public slots:
 	void settingChanged ();
 	void browseRequest (QStringList* strings);
-	/** Invoke the plugin download dialog. */
-	void downloadPlugins ();
 private:
 	MultiStringSelector *map_choser;
 	QButtonGroup *button_group;
 	QCheckBox *show_code_box;
 	RKSpinBox *code_size_box;
-	
+
+	/** active plugin maps */
 	static QStringList plugin_maps;
+	/** plugin maps which are not necessarily active, but have been encountered, before. @see plugin_maps */
+	static QStringList known_plugin_maps;
 	static PluginPrefs interface_pref;
 	static bool show_code;
 	static int code_size;
 
+/* TODO: These are currently unused (leftovers of GHNS-based plugin installation), but might still be of interest */
 	static void installPluginPack (const QString &archive_file);
 	static void uninstallPluginPack (const QString &archive_file);
 	static QString baseNameOfPluginPack (const QString &archive_file);
 	static QStringList findPluginMapsRecursive (const QString &basedir);
+	static void fixPluginMapLists ();
 };
 
 #endif
