@@ -116,14 +116,10 @@ RKCommandEditorWindow::RKCommandEditorWindow (QWidget *parent, bool use_r_highli
 	connect (m_doc, SIGNAL (documentUrlChanged (KTextEditor::Document*)), this, SLOT (updateCaption (KTextEditor::Document*)));
 	connect (m_doc, SIGNAL (modifiedChanged (KTextEditor::Document*)), this, SLOT (updateCaption(KTextEditor::Document*)));                // of course most of the time this causes a redundant call to updateCaption. Not if a modification is undone, however.
 	connect (m_doc, SIGNAL (modifiedChanged (KTextEditor::Document*)), this, SLOT (autoSaveHandlerModifiedChanged()));
-	connect (m_doc, SIGNAL (textChanged (KTextEditor::Document*)), this, SLOT (tryCompletionProxy (KTextEditor::Document*)));
 	connect (m_doc, SIGNAL (textChanged (KTextEditor::Document*)), this, SLOT (autoSaveHandlerTextChanged()));
 	connect (m_view, SIGNAL (selectionChanged(KTextEditor::View*)), this, SLOT (selectionChanged(KTextEditor::View*)));
 	// somehow the katepart loses the context menu each time it loses focus
 	connect (m_view, SIGNAL (focusIn(KTextEditor::View*)), this, SLOT (focusIn(KTextEditor::View*)));
-	completion_timer = new QTimer (this);
-	completion_timer->setSingleShot (true);
-	connect (completion_timer, SIGNAL (timeout ()), this, SLOT (tryCompletion()));
 
 	completion_model = 0;
 	cc_iface = 0;
@@ -134,6 +130,10 @@ RKCommandEditorWindow::RKCommandEditorWindow (QWidget *parent, bool use_r_highli
 		if (cc_iface) {
 			cc_iface->setAutomaticInvocationEnabled (true);
 			completion_model = new RKCodeCompletionModel (this);
+			completion_timer = new QTimer (this);
+			completion_timer->setSingleShot (true);
+			connect (completion_timer, SIGNAL (timeout ()), this, SLOT (tryCompletion()));
+			connect (m_doc, SIGNAL (textChanged (KTextEditor::Document*)), this, SLOT (tryCompletionProxy (KTextEditor::Document*)));
 		} else {
 			RK_ASSERT (false);
 		}
@@ -193,6 +193,10 @@ void RKCommandEditorWindow::initializeActions (KActionCollection* ac) {
 	action_run_selection = RKStandardActions::runSelection (this, this, SLOT (runSelection()));
 	action_run_selection->setEnabled (false);
 	action_run_line = RKStandardActions::runLine (this, this, SLOT (runLine()));
+	// NOTE: enter_and_submit is not currently added to the menu
+	KAction *action = ac->addAction ("enter_and_submit", this, SLOT (enterAndSubmit()));
+	action->setText (i18n ("Insert line break and run"));
+	action->setShortcuts (KShortcut (Qt::ControlModifier + Qt::Key_Return, Qt::ControlModifier + Qt::Key_Enter));
 
 	action_help_function = RKStandardActions::functionHelp (this, this, SLOT (showHelp()));
 
@@ -509,7 +513,7 @@ void RKCommandEditorWindow::tryCompletionProxy (KTextEditor::Document*) {
 	if (RKSettingsModuleCommandEditor::completionEnabled ()) {
 		if (cc_iface && cc_iface->isCompletionActive ()) {
 			tryCompletion ();
-		} else {
+		} else if (cc_iface) {
 			completion_timer->start (RKSettingsModuleCommandEditor::completionTimeout ());
 		}
 	}
@@ -623,6 +627,16 @@ void RKCommandEditorWindow::runLine() {
 	m_view->setCursorPosition (c);
 }
 
+void RKCommandEditorWindow::enterAndSubmit () {
+	RK_TRACE (COMMANDEDITOR);
+
+	KTextEditor::Cursor c = m_view->cursorPosition ();
+	int line = c.line ();
+	m_doc->insertText (c, "\n");
+	QString command = m_doc->line (line);
+	if (!command.isEmpty ()) RKConsole::pipeUserCommand (command + '\n');
+}
+
 void RKCommandEditorWindow::copyLinesToOutput () {
 	RK_TRACE (COMMANDEDITOR);
 
@@ -795,7 +809,7 @@ RKFunctionArgHinter::RKFunctionArgHinter (RKScriptContextProvider *provider, KTe
 	arghints_popup->setMargin (2);
 	QPalette p = QToolTip::palette ();		// HACK to trick the style into using the correct color
 	p.setColor (QPalette::Inactive, QPalette::Window, p.color (QPalette::Inactive, QPalette::ToolTipBase));
-	p.setColor (QPalette::Inactive, QPalette::Text, p.color (QPalette::Inactive, QPalette::ToolTipText));
+	p.setColor (QPalette::Inactive, QPalette::WindowText, p.color (QPalette::Inactive, QPalette::ToolTipText));
 	arghints_popup->setPalette (p);
 	arghints_popup->setFrameStyle (QFrame::Box);
 	arghints_popup->setLineWidth (1);
@@ -1123,16 +1137,17 @@ QString RKCommandHighlighter::commandToHTML (const QString r_command, Highlighti
 		}
 
 		int handledUntil = lineStart;
-		int remainingChars = line.length() - handledUntil;
+		int remainingChars = line.length();
 		foreach ( const KTextEditor::HighlightInterface::AttributeBlock& block, attribs ) {
 			if ((block.start + block.length) <= handledUntil) continue;
 			int start = qMax(block.start, lineStart);
 			if ( start > handledUntil ) {
 				ret += exportText( line.mid( handledUntil, start - handledUntil ), noAttrib, m_defaultAttribute );
 			}
-			int length = qMin(block.length, remainingChars);
+			int end = qMin (block.start + block.length, remainingChars);
+			int length = end - start;
 			ret += exportText( line.mid( start, length ), block.attribute, m_defaultAttribute);
-			handledUntil = start + length;
+			handledUntil = end;
 		}
 
 		if ( handledUntil < lineStart + remainingChars ) {
