@@ -256,12 +256,24 @@ void RKMDIWindow::windowActivationChange (bool) {
 	if (active || (!isAttached ())) update ();
 }
 
+void RKMDIWindow::slotActivate () {
+	RK_TRACE (APP);
+
+	activate (true);
+}
+
 void RKMDIWindow::enterEvent (QEvent *event) {
 	RK_TRACE (APP);
 
 	if (!isActive ()) {
 		if (RKSettingsModuleGeneral::mdiFocusPolicy () == RKSettingsModuleGeneral::RKMDIFocusFollowsMouse) {
-			activate (true);
+			if (!QApplication::activePopupWidget ()) {
+				// see https://sourceforge.net/tracker/?func=detail&atid=459007&aid=3327985&group_id=50231
+				// enter events may be delivered while a popup-menu (in a different window) is executing. If we activate in this case, the popup-menu might get deleted
+				// while still handling events. Similar problems seem to occur, when the popup menu has just finished (by the user selecting an action) and this results
+				// in the mouse entering this widget. To prevent crashes in this second case, we delay the activation until the next iteration of the event loop.
+				QTimer::singleShot (0, this, SLOT (slotActivate()));
+			}
 		}
 	}
 
