@@ -130,6 +130,8 @@ protected:
 friend class RKWorkplace;
 /** type of this window */
 	int type;
+private slots:
+	void slotActivate ();
 private:
 friend class RKToolWindowBar;
 /** state of this window (attached / detached). This is usually set from the RKWorkplace */
