@@ -2,7 +2,7 @@
                           rktoplevelwindowgui  -  description
                              -------------------
     begin                : Tue Apr 24 2007
-    copyright            : (C) 2007, 2009 by Thomas Friedrichsmeier
+    copyright            : (C) 2007, 2009, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -38,6 +38,8 @@
 #include "../windows/rkhelpsearchwindow.h"
 #include "../windows/rkmdiwindow.h"
 #include "../misc/rkstandardicons.h"
+#include "../misc/rkprogresscontrol.h"
+#include "../misc/rkcommonfunctions.h"
 #include "../plugin/rkcomponentmap.h"
 #include "../rbackend/rinterface.h"
 #include "../rkglobals.h"
@@ -69,31 +71,24 @@ RKTopLevelWindowGUI::RKTopLevelWindowGUI (KXmlGuiWindow *for_window) : QObject (
 	show_rkward_help->setStatusTip (i18n ("Show help on RKWard"));
 
 	// window menu
+	// NOTE: enabling / disabling the prev/next actions is not a good idea. It will cause the script windows to "accept" their shortcuts, when disabled
+	prev_action = actionCollection ()->addAction ("prev_window", this, SLOT (previousWindow()));
+	prev_action->setText (i18n ("Previous Window"));
+	prev_action->setIcon (QIcon (RKCommonFunctions::getRKWardDataDir () + "icons/window_back.png"));
+	prev_action->setShortcut (Qt::ControlModifier + Qt::Key_Tab);
+	next_action = actionCollection ()->addAction ("next_window", this, SLOT (nextWindow()));
+	next_action->setText (i18n ("Next Window"));
+	next_action->setIcon (QIcon (RKCommonFunctions::getRKWardDataDir () + "icons/window_forward.png"));
+	next_action->setShortcut (Qt::ControlModifier + Qt::ShiftModifier + Qt::Key_Tab);
+
 	KAction *action;
-	action = actionCollection ()->addAction ("window_show_workspace", this, SLOT(toggleWorkspace()));
-	action->setText (i18n ("Show/Hide Workspace Browser"));
-	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowWorkspaceBrowser));
-	action->setShortcut (Qt::AltModifier + Qt::Key_1);
-	action = actionCollection ()->addAction ("window_show_filebrowser", this, SLOT(toggleFilebrowser()));
-	action->setText (i18n ("Show/Hide Filesystem Browser"));
-	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowFileBrowser));
-	action->setShortcut (Qt::AltModifier + Qt::Key_2);
-	action = actionCollection ()->addAction ("window_show_commandlog", this, SLOT(toggleCommandLog()));
-	action->setText (i18n ("Show/Hide Command Log"));
-	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowCommandLog));
-	action->setShortcut (Qt::AltModifier + Qt::Key_3);
-	action = actionCollection ()->addAction ("window_show_pendingjobs", this, SLOT(togglePendingJobs()));
-	action->setText (i18n ("Show/Hide Pending Jobs"));
-	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowPendingJobs));
-	action->setShortcut (Qt::AltModifier + Qt::Key_4);
-	action = actionCollection ()->addAction ("window_show_console", this, SLOT(toggleConsole()));
-	action->setText (i18n ("Show/Hide Console"));
-	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowConsole));
-	action->setShortcut (Qt::AltModifier + Qt::Key_5);
-	action = actionCollection ()->addAction ("window_show_helpsearch", this, SLOT(toggleHelpSearch()));
-	action->setText (i18n ("Show/Hide R Help Search"));
-	action->setIcon (RKStandardIcons::getIcon (RKStandardIcons::WindowSearchHelp));
-	action->setShortcut (Qt::AltModifier + Qt::Key_6);
+	foreach (RKToolWindowList::ToolWindowRepresentation rep, RKToolWindowList::registeredToolWindows ()) {
+		action = actionCollection ()->addAction ("window_show_" + rep.id, this, SLOT (toggleToolView()));
+		action->setText (i18n ("Show/Hide %1", rep.window->shortCaption ()));
+		action->setIcon (rep.window->windowIcon ());
+		action->setShortcut (rep.default_shortcut);
+		action->setData (rep.id);
+	}
 	action = actionCollection ()->addAction ("window_activate_docview", this, SLOT(activateDocumentView()));
 	action->setText (i18n ("Activate Document view"));
 	action->setShortcut (Qt::AltModifier + Qt::Key_0);
@@ -148,18 +143,47 @@ void RKTopLevelWindowGUI::startWhatsThis () {
 void RKTopLevelWindowGUI::reportRKWardBug () {
 	RK_TRACE (APP);
 
-// TOOD: something pretty
-	KMessageBox::information (for_window, i18n ("<p>Please submit your bug reports or wishes at <a href=\"%1\">%1</a> or send email to <a href=\"mailto:%2\">%2</a>.</p>"
-							, QString ("http://sourceforge.net/tracker/?group_id=50231&atid=459007"), QString ("rkward-devel@lists.sourceforge.net")),
-							i18n ("Reporting bugs in RKWard"), QString (), KMessageBox::Notify | KMessageBox::AllowLink);
+	QString report_template = i18n ("---Problem description---\n");
+	report_template.append (i18n ("Please give a brief summary on the problem:\n###Please fill in###\n\n"));
+	report_template.append (i18n ("What - in detail - did you do directly before you encountered this problem?\n###Please fill in###\n\n"));
+	report_template.append (i18n ("When you try to repeat the above, does the problem occur again (no, sometimes, always)?\n###Please fill in###\n\n"));
+	report_template.append (i18n ("If applicable: When doing the same thing in an R session outside of RKWard, do you see the same problem?\n###Please fill in###\n\n"));
+	report_template.append (i18n ("Do you have any further information that might help us to track this problem down? In particular, if applicable, can you provide sample data and sample R code to reproduce this problem?\n###Please fill in###\n\n"));
+
+	RCommand *command = new RCommand ("rk.sessionInfo()", RCommand::App);
+	RKProgressControl *control = new RKProgressControl (this, i18n ("Please stand by while gathering some information on your setup.\nIn case the backend has died or hung up, you may want to press 'Cancel' to skip this step."), i18n ("Gathering setup information"), RKProgressControl::CancellableNoProgress);
+	control->addRCommand (command, true);
+	RKGlobals::rInterface ()->issueCommand (command);
+	bool ok = control->doModal (false);
+
+	report_template.append ("---Session Info---\n");
+	report_template.append (control->fullCommandOutput ());
+	if (!ok) report_template.append ("- not available -");
+	delete control;
+
+	KDialog *dialog = new KDialog (for_window);
+	connect (dialog, SIGNAL (finished(int)), dialog, SLOT (deleteLater()));
+	dialog->setCaption (i18n ("Reporting bugs in RKWard"));
+	dialog->setButtons (KDialog::Ok);
+	KVBox *vbox = new KVBox (dialog);
+	dialog->setMainWidget (vbox);
+	QLabel *label = new QLabel (i18n ("<p><b>Where should I report bugs or wishes?</b></p><p>Please submit your bug reports or wishes at <a href=\"%1\">%1</a> or send email to <a href=\"mailto:%2\">%2</a>.</p>"
+							"<p><b>What information should I provide?</b></p><p>Please copy the information shown below, and fill in the details to the questions.</p>"
+							, QString ("http://sourceforge.net/tracker/?group_id=50231&atid=459007"), QString ("rkward-devel@lists.sourceforge.net")), vbox);
+	label->setWordWrap (true);
+	label->setOpenExternalLinks (true);
+	QTextEdit *details = new QTextEdit (vbox);
+	details->setReadOnly (true);
+	details->setText (report_template);
+
+	dialog->show ();
 }
 
 void RKTopLevelWindowGUI::showAboutApplication () {
 	RK_TRACE (APP);
 
-	KAboutApplicationDialog *about = new KAboutApplicationDialog (KCmdLineArgs::aboutData ());
-	about->exec ();
-	delete about;
+	KAboutApplicationDialog about (KCmdLineArgs::aboutData ());
+	about.exec ();
 }
 
 void RKTopLevelWindowGUI::toggleToolView (RKMDIWindow *tool_window) {
@@ -174,6 +198,16 @@ void RKTopLevelWindowGUI::toggleToolView (RKMDIWindow *tool_window) {
 	}
 }
 
+void RKTopLevelWindowGUI::toggleToolView () {
+	RK_TRACE (APP);
+	QAction *act = dynamic_cast<QAction*> (sender ());
+	RK_ASSERT (act);
+
+	RKMDIWindow *win = RKToolWindowList::findToolWindowById (act->data ().toString ());
+	RK_ASSERT (win);
+	toggleToolView (win);
+}
+
 void RKTopLevelWindowGUI::showHelpSearch () {
 	RK_TRACE (APP);
 
@@ -186,42 +220,6 @@ void RKTopLevelWindowGUI::showRKWardHelp () {
 	RKWorkplace::mainWorkplace ()->openHelpWindow (KUrl ("rkward://page/rkward_welcome"), true);
 }
 
-void RKTopLevelWindowGUI::toggleHelpSearch () {
-	RK_TRACE (APP);
-
-	toggleToolView (RKHelpSearchWindow::mainHelpSearch ());
-}
-
-void RKTopLevelWindowGUI::toggleConsole () {
-	RK_TRACE (APP);
-
-	toggleToolView (RKConsole::mainConsole ());
-}
-
-void RKTopLevelWindowGUI::toggleCommandLog () {
-	RK_TRACE (APP);
-
-	toggleToolView (RKCommandLog::getLog ());
-}
-
-void RKTopLevelWindowGUI::togglePendingJobs () {
-	RK_TRACE (APP);
-
-	toggleToolView (RControlWindow::getControl ());
-}
-
-void RKTopLevelWindowGUI::toggleWorkspace () {
-	RK_TRACE (APP);
-
-	toggleToolView (RObjectBrowser::mainBrowser ());
-}
-
-void RKTopLevelWindowGUI::toggleFilebrowser () {
-	RK_TRACE (APP);
-
-	toggleToolView (RKFileBrowser::getMainBrowser ());
-}
-
 void RKTopLevelWindowGUI::activateDocumentView () {
 	RK_TRACE (APP);
 
@@ -235,4 +233,17 @@ void RKTopLevelWindowGUI::slotOutputShow () {
 	RKWorkplace::mainWorkplace ()->openOutputWindow (KUrl ());
 }
 
+void RKTopLevelWindowGUI::nextWindow () {
+	RK_TRACE (APP);
+
+	// well, this is sort of cumbersome, but the switcher widget gets keyboard focus, and so we need to register the window switching actions with it.
+	RKWorkplace::getHistory ()->next (prev_action, next_action);
+}
+
+void RKTopLevelWindowGUI::previousWindow () {
+	RK_TRACE (APP);
+
+	RKWorkplace::getHistory ()->prev (prev_action, next_action);
+}
+
 #include "rktoplevelwindowgui.moc"
