@@ -119,7 +119,6 @@ void RKWorkplaceView::addWindow (RKMDIWindow *widget) {
 	RK_TRACE (APP);
 
 	int id = -1;
-	setUpdatesEnabled (false);
 
 	QIcon icon = widget->windowIcon ();
 	if (icon.isNull ()) icon = widget->topLevelWidget ()->windowIcon ();
@@ -133,8 +132,6 @@ void RKWorkplaceView::addWindow (RKMDIWindow *widget) {
 	if (count () > 1) setTabBarHidden (false);
 
 	setCurrentIndex (id);		// activate the new tab
-
-	setUpdatesEnabled (true);
 }
 
 bool RKWorkplaceView::hasWindow (RKMDIWindow *widget) {
@@ -144,24 +141,12 @@ bool RKWorkplaceView::hasWindow (RKMDIWindow *widget) {
 void RKWorkplaceView::removeWindow (RKMDIWindow *widget, bool destroyed) {
 	RK_TRACE (APP);
 
-	setUpdatesEnabled (false);
-
 	int id = indexOf (widget);		// which page is it?
 	RK_DO (if (id == -1) qDebug ("did not find page in RKWorkplaceView::removeWindow"), APP, DL_WARNING);
 	if (!destroyed) disconnect (widget, SIGNAL (captionChanged (RKMDIWindow *)), this, SLOT (childCaptionChanged (RKMDIWindow *)));
 
 	removeTab (id);
-	int new_count = count ();
-	if (new_count <= 1) {
-		setTabBarHidden (true);
-		if (new_count < 1) {
-			// KDE4: is this still needed?
-			setCaption (QString ());
-			emit (currentChanged (-1));
-		}
-	}
-
-	setUpdatesEnabled (true);
+	if (count () <= 1) setTabBarHidden (true);
 }
 
 // KDE4 TODO: we can use setCurrentWidget, instead.
