@@ -2,7 +2,7 @@
                           robjectbrowser  -  description
                              -------------------
     begin                : Thu Aug 19 2004
-    copyright            : (C) 2004, 2006, 2007, 2008, 2009, 2010 by Thomas Friedrichsmeier
+    copyright            : (C) 2004, 2006, 2007, 2008, 2009, 2010, 2011 by Thomas Friedrichsmeier
     email                : tfry@users.sourceforge.net
  ***************************************************************************/
 
@@ -65,7 +65,7 @@ RObjectBrowser::RObjectBrowser (QWidget *parent, bool tool_window, const char *n
 	setMetaInfo (i18n ("R workspace browser"), "rkward://page/rkward_workspace_browser", RKSettings::PageObjectBrowser);
 	initializeActivationSignals ();
 
-	setCaption (i18n ("Objects in the R workspace"));
+	setCaption (i18n ("R Workspace"));
 }
 
 RObjectBrowser::~RObjectBrowser () {
@@ -233,7 +233,7 @@ void RObjectBrowserInternal::popupRename () {
 	QString name = KInputDialog::getText (i18n ("Rename object"), i18n ("Enter the new name"), list_view->menuObject ()->getShortName (), &ok, this);
 	
 	if (ok) {
-		QString valid = list_view->menuObject ()->getContainer ()->validizeName (name);
+		QString valid = static_cast<RContainerObject*> (list_view->menuObject ()->parentObject ())->validizeName (name);
 		if (valid != name) KMessageBox::sorry (this, i18n ("The name you specified was already in use or not valid. Renamed to %1", valid), i18n ("Invalid Name"));
 		RKGlobals::tracker ()->renameObject (list_view->menuObject (), valid);
 	}
