@@ -1,9 +1,3 @@
-## intro
-# This should be the first line in each test suite file: Include the
-# test framework, unless already included (multiple inclusion would not
-# really do any harm either, though
-if (!isClass ("RKTestSuite")) source ("test_framework.R")
-
 ## definition of the test suite
 suite <- new ("RKTestSuite", id="import_export_plugins",
 	# place here libraries that are required for *all* tests in this suite, or highly likely to be installed
@@ -13,22 +7,21 @@ suite <- new ("RKTestSuite", id="import_export_plugins",
 		function () {
 			# prepare some different files for loading
 			library ("datasets")
-			women <- datasets::women
-
-			save (women, file="women.RData")
-			write.csv (women, file="women.csv")
-
-			suppressWarnings (rm ("women"))
+			assign("women.data", datasets::women, pos=globalenv())
 		}
 	## the tests
 	), tests = list (
 		new ("RKTest", id="load_r_object", call=function () {
+			save (women.data, file="women.RData")
+
 			rk.call.plugin ("rkward::load_r_object", file.selection="women.RData", envir.active="0", submit.mode="submit")
 
-			stopifnot (all.equal (.GlobalEnv$women, datasets::women))
+			stopifnot (all.equal (.GlobalEnv$women.data, datasets::women))
 		}),
 		new ("RKTest", id="import_csv", call=function () {
-			rk.call.plugin ("rkward::import_csv", allow_escapes.state="", blanklinesskip.state="TRUE", checkname.state="TRUE", colclass.string="", colname.string="", dec.string="'.'", doedit.state="0", file.selection="women.csv", flush.state="", isrow.state="true", na.text="NA", name.objectname="women", nrows.text="-1", quick.string="csv", quote.string="'\\\"'", sep.string="','", skip.text="0", strings_as_factors.string="", stripwhite.state="FALSE", rowname.string="rowcol", nomrow.text="1", submit.mode="submit")
+			write.csv (women.data, file="women.csv")
+
+			rk.call.plugin ("rkward::import_csv", allow_escapes.state="", blanklinesskip.state="TRUE", checkname.state="TRUE", colclass.string="", colname.string="", dec.string="'.'", doedit.state="0", file.selection="women.csv", flush.state="", na.text="NA", name.objectname="women", nrows.text="-1", quick.string="csv", quote.string="'\\\"'", sep.string="','", skip.text="0", strings_as_factors.string="", stripwhite.state="FALSE", rowname.string="rowcol", nomrow.text="1", submit.mode="submit")
 
 			stopifnot (all.equal (.GlobalEnv$women, datasets::women))
 		}),
@@ -51,7 +44,7 @@ suite <- new ("RKTestSuite", id="import_export_plugins",
 			stopifnot (oldwd == getwd ())
 		}),
 		new ("RKTest", id="import_spss", call=function () {
-			rk.call.plugin ("rkward::import_spss", convert_var_labels.state="1", data_frame.state="1", do_locale_conversion.state="0", doedit.state="0", file.selection="../import_export_plugins_testfile.sav", labels_limit.real="1.00", saveto.objectname="my.spss.data", trim_labels.state="0", use_labels.state="1", submit.mode="submit")
+			rk.call.plugin ("rkward::import_spss", convert_var_labels.state="1", data_frame.state="1", do_locale_conversion.state="0", doedit.state="0", file.selection="import_export_plugins_testfile.sav", labels_limit.real="1.00", saveto.objectname="my.spss.data", trim_labels.state="0", use_labels.state="1", submit.mode="submit")
 
 			# In order to check, whether the import was correct
 			rk.print (my.spss.data)
@@ -59,14 +52,14 @@ suite <- new ("RKTestSuite", id="import_export_plugins",
 
 			# WARNING: TODO: We don't use the value labels of the third
 			# variable, yet.
-		}, libraries=c("foreign")),
+		}, libraries=c("foreign"), files=c("../import_export_plugins_testfile.sav")),
 		new ("RKTest", id="import_stata", call=function () {
-			rk.call.plugin ("rkward::import_stata", convert_dates.state="1", convert_factors.state="1", convert_underscore.state="0", doedit.state="0", file.selection="../import_export_plugins_testfile.dta", missing_type.state="0", saveto.objectname="my.stata.data", submit.mode="submit")
+			rk.call.plugin ("rkward::import_stata", convert_dates.state="1", convert_factors.state="1", convert_underscore.state="0", doedit.state="0", file.selection="import_export_plugins_testfile.dta", missing_type.state="0", saveto.objectname="my.stata.data", submit.mode="submit")
 
 			# In order to check, whether the import was correct
 			rk.print (my.stata.data)
 			for (var in my.stata.data) rk.print (rk.get.description(var))
-		}, libraries=c("foreign")),
+		}, libraries=c("foreign"), files=c("../import_export_plugins_testfile.dta")),
 		new ("RKTest", id="load_source", call=function () {
 			stopifnot (!exists ("testx", globalenv ()))
 
@@ -114,11 +107,15 @@ suite <- new ("RKTestSuite", id="import_export_plugins",
 			for (line in x) rk.print (line)
 		}),
 		new ("RKTest", id="package_skeleton", call=function () {
-			rk.call.plugin ("rkward::save_skeleton", data.available="rktest.setSuiteStandards\nrktest.runRKTestSuite", force.state="TRUE", name.text="anRpackage", path.selection=".", submit.mode="submit")
-		})
-	), postCalls = list ()	# like initCalls: run after all tests to clean up. Empty in this case.
-)
+			# create two functions to use
+			assign ("skel.func1", rkwardtests::rktest.getTempDir, envir=globalenv())
+			assign ("skel.func2", rkwardtests::rktest.getTempDir, envir=globalenv())
+			rk.sync.global()
 
-## always store the result in "results" and print it
-results <- rktest.runRKTestSuite (suite)
-print (results)
+			rk.call.plugin ("rkward::save_skeleton", data.available="skel.func1\nskel.func2", force.state="TRUE", name.text="anRpackage", path.selection=".", submit.mode="submit")
+			rm (skel.func1, skel.func2, envir=globalenv())
+		})
+	), postCalls = list (
+			function(){rm("women.data", pos=globalenv())}
+			)	# like initCalls: run after all tests to clean up. Empty in this case.
+)
